/**CFile*******************************************************************
  PackageName [ccs]
  Synopsis    [Package 'ccs' provides an extended CCS parser.]

  FileName    [ccsParser.y]
  Revision    [$Revision: 2 $]
  Date        [$Date: 2006-10-05 15:30:29 +0200 (Thu, 05 Oct 2006) $]
  Authors     [Robert Meolic (meolic@uni-mb.si)]
  Description [File ccsParser.y is a bison file for CCS parser.]
  SeeAlso     [ccs.h, ccsInt.h, ccsParser.l]

  Copyright   [see ccs.h]
  ************************************************************************/

%{
#include "ccsInt.h"
#define YYERROR_VERBOSE 
%}
	
%{
   typedef struct {
     int act;
     int type;
   } typedAction;
   typedef struct {
     typedAction *set;
     int len;
   } actionTrace;
%}

%{
   static void outline(char *s);
   static void lowercase(char *s);
   static void addToSet(int **set, int *len, int state);
   static void addToTrace(typedAction **set, int *len, int act, int type);
   static void addToTraceSet(actionTrace **set, int *len, actionTrace trace);
   static void writeSet(int *set, int len);
   static void writeTraceSet(actionTrace *set, int len);
%}

%{
   /* UNEXPECTED SIDEEFFECT ON SOME SYSTEMS */
   /* THE VARIABLES DEFINED HERE ARE NOT ALWAYS LOCAL, THEY CAN BE GLOBAL! */
   /* THEREFORE, I USE A PREFIX FOR EACH VARIABLE */

   /* STATIC VARIABLES? I DO NOT KNOW, IF THIS IS A GOOD OR A BAD IDEA */
   /*
   static int CCSSort;
   static int CCSProc;
   static Bdd_String CCSsortname;
   static Bdd_String CCSLowerProc;
   static Bdd_String CCSPlist;
   static Bdd_String CCSAlist;
   static Bdd_String CCSOld,CCSNew;
   static int CCSNum,CCSNump,CCSNuma;
   */

   int CCSSort;
   int CCSProc;
   Bdd_String CCSinputfile;
   Bdd_String CCSsortname;
   Bdd_String CCSLowerProc;
   Bdd_String CCSPlist;
   Bdd_String CCSAlist;
   Bdd_String CCSOld,CCSNew;
   int CCSNum,CCSNump,CCSNuma,CCSCompType;
%}

%union {
  Bdd_String outline;
  Bdd_String word;
  Bdd_String line;
  struct {
    int *set;
    int len;
  } setOfStates;
  actionTrace traceOfActions;
  struct {
    actionTrace *set;
    int len;
  } setOfTraces;
}


%token UNKNOWN
%token <outline> OUTLINE
%token <word> PROCESS NAME ACTION SEP NET INTERLEAVE SYNCHRONOUS COMPOSE STARTPARTIAL ENDPARTIAL
%token <word> PROPERTY AUTOMATON PRODUCT SYNPROD
%token <line> FORMULA
%type <setOfStates> procbody
%type <traceOfActions> onetrace
%type <setOfTraces> tracelist

%left '+'
%left SEP

%%
  start		: body ;

  body          : outline body {}
                | process body {}
                | PROCESS process body {}
                | NET net body {}
                | PROPERTY property body {} 
                | AUTOMATON automaton body {} 
                | PRODUCT product body {}
                | ;

  outline       : OUTLINE {

                    Bdd_String tmp;

                    tmp = strdup($1);
                    outline(tmp);
                    free(tmp);
                    free($1);
                  };

  property      : NAME FORMULA {

                    Bdd_String name;
                    Bdd_String formula,tmp;
                    int i,len;

                    name = strdup($1);
                    printf("  Property %s\n",name);

                    i = strspn(&$2[2]," ");
                    formula = strdup(&$2[i+2]);

                    /*
                    fprintf(stderr,"FORMULA NAME = %s\n",name);
                    fprintf(stderr,"FORMULA BODY = %s\n",formula);
	            */

       	            Mc_ReadACTL(name,formula);

                    i = Mc_FindFormula(name);
                    if (i != -1) {
                      printf("    %s = %s\n",name,mc_formulaTable[i].formula);
		    }

		    free(name);
                    free(formula);
                    free($1);
                    free($2);
                  };

  automaton     : NAME '=' NAME '@' NAME {

                    Bdd_String aname,fname,pname;

                    aname = strdup($1);
                    fname = strdup($3);
                    pname = strdup($5);

                    printf("  Automaton %s\n",aname);
                    printf("    Creating automaton %s @ %s ... ",fname,pname);

                    Mc_GenerateWCA(pname,fname,aname);

                    printf("OK\n");

		    free(aname);
		    free(fname);
		    free(pname);
                    free($1);
                    free($3);
                    free($5);
                  } ;

  product       : NAME {

                    Bdd_String name;

		    /*
                    fprintf(stderr,"PRODUCT NAME %s\n",$1);
		    */

                    name = strdup($1);
                    printf("  Product %s\n",name);

                    CCSNuma = 0;
                    CCSAlist = NULL;

                    free(name);
                  }

                  '=' '(' NAME SYNPROD NAME ')' restrlist {

                    Bdd_String name,pname,aname;

                    name = strdup($1);
                    pname = strdup($5);
                    aname = strdup($7);

                    Strucval_SyncProduct(name,pname,aname,CCSAlist);

		    /*
                    printf("OK\n");
		    */

                    free(name);
                    free(pname);
                    free(aname);
                    free(CCSAlist);
                    free($1);
                    free($5);
                    free($7);
		  };

  process	: NAME {  

                    Bdd_String name;

                    /*
                    fprintf(stderr,"NAME %s\n",$1);
		    */

                    name = strdup($1);
                    printf("  Process %s ... ",name);

                    CCSProc = Pa_AddNewProcess(name);
                    CCSLowerProc = strdup(name);
                    lowercase(CCSLowerProc);

		    CCSsortname = NULL;

                    free(name);
		  }
                  sort {
		    if (!CCSsortname) CCSsortname = strdup(CCSinputfile);
                    CCSSort = Pa_FindSort(CCSsortname);
                    if (CCSSort == -1) {
                      CCSSort = Pa_AddNewSort(CCSsortname);
		    }
                    pa_processTable[CCSProc].sort = CCSSort;
                    free(CCSsortname);
                  }
                  '='
                  procbody {

		    int i,j;
                    int state;
                    Bdd_String tmp;
                    Bdd_Boolean OK;

                    OK = TRUE;

 		    if ($6.len == 1) {
                      pa_processTable[CCSProc].initial = $6.set[0];
		    } else {

                      for (i=0; i<$6.len; i++) {
                        tmp = strdup(pa_processTable[CCSProc].tableS[$6.set[i]].name);
                        if (!strchr(tmp,'<')) {
                          OK = FALSE;
		        }
                        free(tmp);
		      }

                      if (!OK) {
                        printf("\n  ERROR in CCS parser: the given usage of operator '+' is not supported, yet!\n");
		      }

                      for (i=0; i<$6.len; i++) {
                        Pa_DeleteStateProcess(&pa_processTable[CCSProc],$6.set[i]);
		      }

                      state = Pa_FOANextStateProcess(&pa_processTable[CCSProc],CCSLowerProc);
                      for (i=0; i<$6.len; i++) {
                        for(j=0; j<pa_processTable[CCSProc].numTransitions; j++) {
                          if (pa_processTable[CCSProc].tableT[j].source == $6.set[i]) {
		            pa_processTable[CCSProc].tableT[j].source = state;
		          }
		        }
		      }
                      pa_processTable[CCSProc].initial = state;
		    }

                    if (OK) {
                      printf("OK\n");
		    }

                    free($1);
                    free($6.set);
                    free(CCSLowerProc);
                  };

  net           : NAME {

                    Bdd_String name;

		    /*
                    fprintf(stderr,"NET NAME %s\n",$1);
		    */

                    name = strdup($1);
                    printf("  Net %s\n",name);
                    printf("    Parallel composition ... ");

                    CCSNump = 0;
                    CCSNuma = 0;
                    CCSPlist = NULL;
                    CCSAlist = NULL;

		    CCSsortname = NULL;

                    free(name);
		  }

                  sort {
		    if (!CCSsortname) CCSsortname = strdup(CCSinputfile);
                    CCSSort = Pa_FindSort(CCSsortname);
                    if (CCSSort == -1) {
                      CCSSort = Pa_AddNewSort(CCSsortname);
		    }
                    pa_processTable[CCSProc].sort = CCSSort;
                    free(CCSsortname);
                  }

                  '=' compbody {

                    Bdd_String comp;

                    comp = strdup($1);

                    if (!pa_sortTable[CCSSort].encoded) {
                      Pa_EncodeSort(pa_sortTable[CCSSort].name);
		    }
                    Pa_EncodeMultiProcesses(CCSPlist);

                    Versis_Compose(comp,CCSCompType,CCSPlist,CCSAlist);

                    printf("OK\n");
                    printf("    Creating process %s ... ",comp);
                    Pa_Composition2Process(comp,TRUE);
                    printf("OK\n");

		    /*
                    printf("\n");
                    Pa_WriteProcessCCS(comp,0);
		    */

                    free(comp);
                    free(CCSPlist);
                    free(CCSAlist);
                    free($1);
                  };

  sort          : '(' NAME ')'{
                    CCSsortname = strdup($2);
                  }
 
                | ;


  procbody      : NAME {

                    int state;
                    Bdd_String name;

                    /*
                    fprintf(stderr,"NAME == %s\n",$1);
		    */

                    name = strdup($1);
                    state = Pa_FOAGateProcess(&pa_processTable[CCSProc],name);

                    $$.set = NULL;
                    $$.len = 0;
                    addToSet(&$$.set,&$$.len,state);

                    free(name);
                    free($1);
		  }

                | procbody '+' procbody {

		    int i;

		    /*
                    fprintf(stderr,"PLUS\n");
		    */

                    $$.set = NULL;
                    $$.len = 0;
                    for (i=0; i<$1.len; i++) {
                      addToSet(&$$.set,&$$.len,($1.set)[i]);
		    }
                    for (i=0; i<$3.len; i++) {
                      addToSet(&$$.set,&$$.len,($3.set)[i]);
		    }

                    free($1.set);
                    free($3.set);
                  }

                | '(' procbody ')' {

		    int i,j;
                    int state;
                    Bdd_String tmp;
                    Bdd_Boolean OK;

                    OK = TRUE;

 		    if ($2.len == 1) {
                      $$.set = $2.set;
                      $$.len = $2.len;
		    } else {

                      for (i=0; i<$2.len; i++) {
                        tmp = strdup(pa_processTable[CCSProc].tableS[$2.set[i]].name);
                        if (!strchr(tmp,'<')) {
                          OK = FALSE;
		        }
                        free(tmp);
		      }

                      if (!OK) {
                        printf("\n  ERROR in CCS parser: the given usage of operator '+' is not supported, yet!\n");
		      }

                      for (i=0; i<$2.len; i++) {
                        Pa_DeleteStateProcess(&pa_processTable[CCSProc],$2.set[i]);
		      }

                      state = Pa_FOANextStateProcess(&pa_processTable[CCSProc],CCSLowerProc);
                      for (i=0; i<$2.len; i++) {
                        for(j=0; j<pa_processTable[CCSProc].numTransitions; j++) {
                          if (pa_processTable[CCSProc].tableT[j].source == $2.set[i]) {
		            pa_processTable[CCSProc].tableT[j].source = state;
		          }
		        }
		      }
                      $$.set = NULL;
                      $$.len = 0;
                      addToSet(&$$.set,&$$.len,state);
                      free($2.set);
		    }

                  }

                | ACTION SEP procbody {

		    int i;
		    int state,len,act,type;
                    Bdd_String action;

		    /*
                    fprintf(stderr,"ACTION %s\n",$1);
                    */

                    type = 0;
                    action = strdup($1);

                    if ((action[0] == '?') || (action[0] == '!')) {

                      /* action with prefix */
                      if (action[0] == '?') {
                        type = 0;
		      }
                      if (action[0] == '!') {
                        type = 1;
		      }
                      act = Pa_FOASortAction(&pa_sortTable[CCSSort],&action[1]);

		    } else {

                      /* action with suffix */
                      len = strlen(action);
                      if (action[len-1] == '?') {
                        type = 0;
                        action[len-1]='\0';
                      } else if (action[len-1] == '!') {
                        type = 1;
                        action[len-1]='\0';
                      }
                      act = Pa_FOASortAction(&pa_sortTable[CCSSort],action);
		    }

                    if ($3.len != 1) {
                      printf("ERROR in CCS parser: problem with operator '+'\n"); 
		    }

                    state = Pa_FOANextStateProcess(&pa_processTable[CCSProc],CCSLowerProc);
                    Pa_FOATransition(&pa_processTable[CCSProc],state,act,type,($3.set)[0]);

                    $$.set = NULL;
                    $$.len = 0;
                    addToSet(&$$.set,&$$.len,state);

		    /*
                    writeSet($$.set,$$.len);
         	    */

                    free($3.set);
                    free(action);
                    free($1);
		  }

                | '(' tracelist ')' SEP procbody {

		    int i,j;
                    int act,type,state1,state2;
                    int *set;
                    int len;

                    if ($5.len != 1) {
                      printf("\n  ERROR in CCS parser: the given usage of operator '+' is not supported, yet!\n");
                    }

                    set=NULL;
                    len=0;
                    for (i=0; i<$2.len; i++) {

                      state2 = $5.set[0];
                      for (j=0; j<$2.set[i].len; j++) {

                        act = $2.set[i].set[j].act;
                        type = $2.set[i].set[j].type;

		        /* DEBUGING */
		        /*
                        printf("%s\n",pa_sortTable[CCSSort].table[act].name);
		        */

                        state1 = Pa_FOANextStateProcess(&pa_processTable[CCSProc],CCSLowerProc);
                        Pa_FOATransition(&pa_processTable[CCSProc],state1,act,type,state2);
                        state2 = state1;
                      }

                      addToSet(&set,&len,state1);
                      free($2.set[i].set);
		    }

                    for (i=0; i<len; i++) {
                      Pa_DeleteStateProcess(&pa_processTable[CCSProc],set[i]);
                    }

                    state1 = Pa_FOANextStateProcess(&pa_processTable[CCSProc],CCSLowerProc);
                    for (i=0; i<len; i++) {
                      for(j=0; j<pa_processTable[CCSProc].numTransitions; j++) {
                        if (pa_processTable[CCSProc].tableT[j].source == set[i]) {
		          pa_processTable[CCSProc].tableT[j].source = state1;
	                }
	              }
	            }

                    $$.set = NULL;
                    $$.len = 0;
                    addToSet(&$$.set,&$$.len,state1);

                    free($2.set);

	 	  } ;

  tracelist     : onetrace '+' tracelist {

                    $$.set = $3.set;
                    $$.len = $3.len;
                    addToTraceSet(&$$.set,&$$.len,$1);

                  }

                | onetrace {

                    $$.set = NULL;
                    $$.len = 0;
                    addToTraceSet(&$$.set,&$$.len,$1);

                 };

  onetrace      : ACTION SEP onetrace {

		    int state,len,act,type;
                    Bdd_String action;

                    type = 0;
                    action = strdup($1);

                    if ((action[0] == '?') || (action[0] == '!')) {

                      /* action with prefix */
                      if (action[0] == '?') {
                        type = 0;
		      }
                      if (action[0] == '!') {
                        type = 1;
		      }
                      act = Pa_FOASortAction(&pa_sortTable[CCSSort],&action[1]);

		    } else {

                      /* action with suffix */
                      len = strlen(action);
                      if (action[len-1] == '?') {
                        type = 0;
                        action[len-1]='\0';
                      } else if (action[len-1] == '!') {
                        type = 1;
                        action[len-1]='\0';
                      }
                      act = Pa_FOASortAction(&pa_sortTable[CCSSort],action);
		    }

                    $$.set = $3.set;
                    $$.len = $3.len;
                    addToTrace(&$$.set,&$$.len,act,type);

                    free(action);
                  }

                | ACTION {

		    int state,len,act,type;
                    Bdd_String action;

                    type = 0;
                    action = strdup($1);

                    if ((action[0] == '?') || (action[0] == '!')) {

                      /* action with prefix */
                      if (action[0] == '?') {
                        type = 0;
		      }
                      if (action[0] == '!') {
                        type = 1;
		      }
                      act = Pa_FOASortAction(&pa_sortTable[CCSSort],&action[1]);

		    } else {

                      /* action with suffix */
                      len = strlen(action);
                      if (action[len-1] == '?') {
                        type = 0;
                        action[len-1]='\0';
                      } else if (action[len-1] == '!') {
                        type = 1;
                        action[len-1]='\0';
                      }
                      act = Pa_FOASortAction(&pa_sortTable[CCSSort],action);
		    }

                    $$.set = NULL;
                    $$.len = 0;
                    addToTrace(&$$.set,&$$.len,act,type);

                    free(action);
                  } ;

  compbody      : COMPOSE '(' proclist ')' restrlist {

                    /* actions not in restrlist go alone and synchronised */
                    /* actions in restrlist go synchronised, only */
                    /* action tau goes alone */

                    /*
		    fprintf(stderr,"COMPOSE:\n");
		    fprintf(stderr,"(nump = %d) %s\n",CCSNump,CCSPlist);
                    */

                    CCSCompType = 0;
		  }


                 | '(' oneproc COMPOSE composein ')' restrlist {

                    /* actions not in restrlist go alone and synchronised */
                    /* actions in restrlist go synchronised, only */
                    /* action tau goes alone */

                    /*
		    fprintf(stderr,"COMPOSE:\n");
		    fprintf(stderr,"(nump = %d) %s\n",CCSNump,CCSPlist);
                    */

                    CCSCompType = 0;
		  }

                | oneproc STARTPARTIAL actionlist ENDPARTIAL oneproc { 

                    /* actions not in actionlist go alone, only */
                    /* actions in actionlist go synchronised, only */
                    /* action tau goes alone */

                    /*
		    fprintf(stderr,"PARTIAL COMPOSITION:\n");
		    fprintf(stderr,"(nump = %d) %s\n",CCSNump,CCSPlist);
		    fprintf(stderr,"(nump = %d) %s\n",CCSNump,CCSAlist);
                    */

                    CCSCompType = 2;
		  } ;

                | INTERLEAVE '(' proclist ')' {

                    /* all actions go alone, only */

                    /*
		    fprintf(stderr,"INTERLEAVE:\n");
		    fprintf(stderr,"(nump = %d) %s\n",CCSNump,CCSPlist);
                    */

                    CCSCompType = 2;

                  }

                | '(' oneproc INTERLEAVE interleavein ')' {

                    /* all actions go alone, only */

                    /*
		    fprintf(stderr,"INTERLEAVE:\n");
		    fprintf(stderr,"(nump = %d) %s\n",CCSNump,CCSPlist);
                    */

                    CCSCompType = 2;

                  }

                | SYNCHRONOUS '(' proclist ')' {

                    /* action tau goes alone */
                    /* all other actions go synchronised, only */

                    /*
		    fprintf(stderr,"SYNCHRONOUS:\n");
		    fprintf(stderr,"(nump = %d) %s\n",CCSNump,CCSPlist);
                    */

                    CCSCompType = 1;

		  }

                | '(' oneproc SYNCHRONOUS synchronousin ')' {

                    /* action tau goes alone */
                    /* all other actions go synchronised, only */

                    /*
		    fprintf(stderr,"SYNCHRONOUS:\n");
		    fprintf(stderr,"(nump = %d) %s\n",CCSNump,CCSPlist);
                    */

                    CCSCompType = 1;

		  } ;

  composein     : oneproc COMPOSE composein 
                | oneproc ;

  interleavein  : oneproc INTERLEAVE interleavein
                | oneproc ;

  synchronousin : oneproc SYNCHRONOUS synchronousin
                | oneproc ;

  actionlist	: oneaction ',' moreactions 
                | oneaction
                | ;

  moreactions	: oneaction ',' moreactions 
                | oneaction ;

  oneaction    : NAME {
                    Bdd_String name;
                    Bdd_String tmp;

                    /*
                    fprintf(stderr,"NAME %s\n",$1);
		    */

                    name = strdup($1);
                    Pa_FOASortAction(&pa_sortTable[CCSSort],name);

                    CCSNuma++;
                    if (!CCSAlist) {
                      CCSAlist = strdup(name);
		    } else {
                      tmp = (Bdd_String) malloc(1024);
                      sprintf(tmp,"%s %s",CCSAlist,name);
                      free(CCSAlist);
                      CCSAlist = strdup(tmp);
                      free(tmp);
		    }

                    free(name);
                    free($1);
                  } ;

  proclist	: oneproc ',' proclist 
                | oneproc ;

  oneproc       : NAME {

                    /*
                    fprintf(stderr,"NAME %s\n",$1);
		    */

                    CCSNum = 0;
                    CCSOld = NULL;
                    CCSNew = NULL;

                  } renlist {

                    Bdd_String name,newproc;
                    Bdd_String tmp;
                    int i;

                    name = strdup($1);

                    i = 0;
                    newproc = NULL;
                    do {
                      i++;
                      if (newproc) free(newproc);
                      newproc = (Bdd_String) malloc(127);
                      sprintf(newproc,"%s%d",name,i);
                    } while (Pa_FindProcess(newproc) != -1);

                    CCSNump++;
                    if (!CCSPlist) {
                      CCSPlist = strdup(newproc);
		    } else {
                      tmp = (Bdd_String) malloc(1024);
                      sprintf(tmp,"%s %s",CCSPlist,newproc);
                      free(CCSPlist);
                      CCSPlist = strdup(tmp);
                      free(tmp);
		    }

                    /*
                    fprintf(stderr,"Renaming old = [%s]\n",CCSOld);
                    fprintf(stderr,"Renaming new = [%s]\n",CCSNew);
		    */

                    Pa_CopyProcess(name,newproc,pa_sortTable[CCSSort].name,CCSOld,CCSNew);

                    free(name);
                    free(newproc);
                    free(CCSOld);
                    free(CCSNew);
                    free($1);
                  };

  renlist       : renaction renlist
                | ;

  renaction     : '['
                  NAME
                  '/'
                  NAME
                  ']' {

                    Bdd_String name1,name2;
                    Bdd_String tmp;

                    name1 = strdup($2);
                    name2 = strdup($4);

                    CCSNum++;

                    Pa_FOASortAction(&pa_sortTable[CCSSort],name1);
                    if (!CCSNew) {
                      CCSNew = strdup(name1);
		    } else {
                      tmp = (Bdd_String) malloc(1024);
                      sprintf(tmp,"%s %s",CCSNew,name1);
                      free(CCSNew);
                      CCSNew = strdup(tmp);
                      free(tmp);
		    }

                    Pa_FOASortAction(&pa_sortTable[CCSSort],name2);
                    if (!CCSOld) {
                      CCSOld = strdup(name2);
		    } else {
                      tmp = (Bdd_String) malloc(1024);
                      sprintf(tmp,"%s %s",CCSOld,name2);
                      free(CCSOld);
                      CCSOld = strdup(tmp);
                      free(tmp);
		    }

                    free(name1);
                    free(name2);

                    free($2);
                    free($4);
                  } ;

  restrlist     : restraction restrlist
                | ;

  restraction   : '\\' NAME {

                    Bdd_String name;
                    Bdd_String tmp;

                    /*
                    fprintf(stderr,"NAME %s\n",$2);
		    */

                    name = strdup($2);
                    Pa_FOASortAction(&pa_sortTable[CCSSort],name);

                    CCSNuma++;
                    if (!CCSAlist) {
                      CCSAlist = strdup(name);
		    } else {
                      tmp = (Bdd_String) malloc(1024);
                      sprintf(tmp,"%s %s",CCSAlist,name);
                      free(CCSAlist);
                      CCSAlist = strdup(tmp);
                      free(tmp);
		    }

                    free(name);
                    free($2);
                  };

%%

#include <lex.ccs.c>

/*-----------------------------------------------------------------------*/
/* Definition of exported functions                                      */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

int
Ccs_ParseCCS(Bdd_String f)
{
  YY_BUFFER_STATE buffer;

  printf("\n");
  yyin = fopen(f,"r");
  if (yyin != NULL) {
    CCSinputfile=strdup(f);

    buffer = yy_create_buffer(yyin,YY_BUF_SIZE);
    yy_switch_to_buffer(buffer);
    yyparse();
    yy_delete_buffer(buffer);

    fclose(yyin);
  } else {
    printf("\n  ERROR: I can not open file %s.\n",f);
  }
  return 0;
}

/*-----------------------------------------------------------------------*/
/* Definition of static functions                                        */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static void
outline(char *s)
{
  if (strcmp(s,"")) printf(&s[1]);
  printf("\n");
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static void
lowercase(char *s)
{
   int  i = 0;

   while ( s[i] )
   {
      s[i] = tolower(s[i]);
      i++;
   }

   return;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static void
addToSet(int **set, int *len, int state)
{
  if (!*set) {
    *set = (int *) malloc (sizeof(int));
    (*set)[0] = state;
    *len = 1;
  } else {
    (*len)++;
    *set = (int *) realloc (*set, (*len) * sizeof(int));
    (*set)[(*len) - 1] = state;
  }
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static void
addToTrace(typedAction **set, int *len, int act, int type)
{
  if (!*set) {
    *set = (typedAction *) malloc (sizeof(typedAction));
    (*set)[0].act = act;
    (*set)[0].type = type;
    *len = 1;
  } else {
    (*len)++;
    *set = (typedAction *) realloc (*set, (*len) * sizeof(typedAction));
    (*set)[(*len) - 1].act = act;
    (*set)[(*len) - 1].type = type;
  }
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static void
addToTraceSet(actionTrace **set, int *len, actionTrace trace)
{
  if (!*set) {
    *set = (actionTrace *) malloc (sizeof(actionTrace));
    (*set)[0] = trace;
    *len = 1;
  } else {
    (*len)++;
    *set = (actionTrace *) realloc (*set, (*len) * sizeof(actionTrace));
    (*set)[(*len) - 1] = trace;
  }
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static void
writeSet(int *set, int len)
{
  int i;

  fprintf(stderr,"SET:");
  for (i=0; i<len; i++) {
    fprintf(stderr," %d",set[i]);
  }
  fprintf(stderr,"\n");
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static void
writeTraceSet(actionTrace *set, int len)
{
  int i,j;

  fprintf(stderr,"TRACE SET:\n");
  for (i=0; i<len; i++) {
    fprintf(stderr,"+\n");
    for (j=0; j<set[i].len; j++) {
      fprintf(stderr," act=%d, type=%d\n",set[i].set[j].act,set[i].set[j].type);
    }
  }
  fprintf(stderr,"\n");
}

/*-----------------------------------------------------------------------*/
/* Definition of parser internal functions                               */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

int
yyerror(char *s)
{
  printf("\n  ERROR: ");
  if (strstr(s,"parse error, unexpected UNKNOWN")) {
     printf("unknown or badly placed symbol '%s'",yytext);
  } else
  if (strcmp(s,"parse error, unexpected UNKNOWN, expecting ACTION")) {
     printf("unknown or badly placed symbol '%s'",yytext);
  } else
  if (!strcmp(s,"parse error, unexpected NAME, expecting ACTION")) {
     printf("incorrect action name '%s', missing ! or ?",yytext);
  } else
  if (!strcmp(s,"parse error, unexpected ACTION, expecting NAME")) {
     printf("incorrect process name '%s'",yytext);
  } else {
    printf("%s",s); 
  }
  printf("\n");
  return 1;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

int
yywrap()
{
  return 1;
}
