/**CFile*******************************************************************
  PackageName [estia]
  Synopsis    [Epistemic reasoning]

  FileName    [estia.h]
  Revision    [$Revision: 2 $]
  Date        [$Date: 2006-10-05 15:30:29 +0200 (Thu, 05 Oct 2006) $]
  Authors     [Robert Meolic (meolic@uni-mb.si), 
	       Marina Bagic (marina.bagic@fer.hr)]
  Description [The file estia.h contains declaration of all external
               data structures.]
  SeeAlso     [estiaInt.h, estiaMain.c]

  Copyright   [Copyright (C) 2005-2006 UM-FERI, UZG-FER
               This program is free software; you can redistribute it
               and/or modify it under the terms of the GNU General Public
               License (version 2) as published by the Free Software
               Foundation.

               This program is distributed in the hope that it will be useful,
               but WITHOUT ANY WARRANTY; without even the implied warranty of
               MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
               GNU General Public License for more details.

               You should have received a copy of the GNU General Public
               License along with this program; if not, write to the Free
               Software Foundation, Inc., 59 Temple Place, Suite 330,
               Boston, MA  02111-1307 USA.]
  ************************************************************************/

#ifndef _ESTIA
#define _ESTIA

#include <pthread.h>
#include <sched.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <stdarg.h>

#include <unistd.h>
#include <math.h>

#include <bdd.h>
#include <pa.h>
#include <versis.h>

/* ON WIN32 THERE HAS TO BE DEFINED ESTWIN32 */
/* ON UNIX THERE HAS TO BE DEFINED ESTUNIX */

#ifdef ESTUNIX
#  ifndef EXTERN
#    define EXTERN extern
#  endif
#endif

#ifdef ESTWIN32
#ifndef TCL
#    ifdef BUILD_ESTIA
#      undef EXTERN
#      define EXTERN __declspec (dllexport)
#    else
#      ifdef USE_ESTIA
#        undef EXTERN
#        define EXTERN __declspec (dllimport)
#      else
#        define EXTERN extern
#      endif
#    endif
#else
#    ifdef BUILD_ESTIA
#      undef TCL_STORAGE_CLASS
#      define TCL_STORAGE_CLASS DLLEXPORT
#    else
#      undef TCL_STORAGE_CLASS
#      define TCL_STORAGE_CLASS DLLIMPORT
#    endif
#endif
#endif

/*-----------------------------------------------------------------------*/
/* Constant declarations                                                 */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Type declarations                                                     */
/*-----------------------------------------------------------------------*/

#ifndef _COMMONTYPE
#define _COMMONTYPE
typedef void *pvoid;
#endif

/*-----------------------------------------------------------------------*/
/* Structure declarations                                                */
/*-----------------------------------------------------------------------*/
typedef struct {
    Bdd_String nameK;
    int numAtoms;
    char *atoms [15];
} buffer;

typedef struct {
    buffer *ptr;
    int numBuffers;
} dK;

typedef struct {
  Bdd_String  name;		/* atom name */
} Estia_Atom;


typedef struct {
  Bdd_String  name;		/* a knowledge name */
  int numAtoms;  
  Estia_Atom *tableA;
} Estia_Knowledge;

typedef struct {
  Bdd_String  name;		/* a state name */
  int numK;
  Estia_Knowledge *tableK;
} Estia_AgentState;

typedef struct {
  int source;		/* a source state */
  int action;		/* an action */
  int type;		/* an action type, 0 = output, 1 = input */
  int dest;		/* a destination state */
} Estia_AgentTransition;

typedef struct {
  Bdd_String     	name;			/* an agent's name */
  Bdd_Boolean    	deleted;        	/* TRUE = deleted */
  int 	                sort;  
  int            	numStates;      	/* a number of states */
  Estia_AgentState    	*tableS;        	/* a table of states */
  int            	initial;        	/* an initial state */
  int            	numTransitions; 	/* a number of transitions */
  Estia_AgentTransition *tableT;        	/* a table of transitions */
  int            	numK;      		/* a number of knowledge instances */
  Estia_Knowledge	*tableK;		/* a table of knowledge instances */

} Estia_Agent;

/*-----------------------------------------------------------------------*/
/* Variable declarations                                                 */
/*-----------------------------------------------------------------------*/
EXTERN int estia_status;

EXTERN int estia_agents;                     /* number of agents */

EXTERN int estia_compositions;               /* number of compositions */

EXTERN int estia_sorts;                      /* number of sorts */

EXTERN Estia_Agent *estia_agentTable;        /* table of agents */


/*-----------------------------------------------------------------------*/
/* Macro declarations                                                    */
/*-----------------------------------------------------------------------*/

/**AutomaticStart*********************************************************/

/*-----------------------------------------------------------------------*/
/* Function prototypes                                                   */
/*-----------------------------------------------------------------------*/

/* void getFromKnowledge(); */	/* operate on ask performative*/

/* void findOrAddKnowledge(); */	/* operate on inform performative*/

/* void addToKnowledge();	*/	/* operate on inform-result performative*/

/* void deleteFromKnowledge(); */	/* operate on untell performative*/

/* void updateKnowledge();	*/	/* operate on update performative*/


/*-----------------------------------------------------------------------*/
/* estiaMain.c                                                        */
/*-----------------------------------------------------------------------*/

EXTERN void Estia_InitPkg();

EXTERN void Estia_ExitPkg();

EXTERN void Estia_AboutPkg();

EXTERN void Estia_DeleteAgent(int num);


/*-----------------------------------------------------------------------*/
/* estiaParser.y                                                         */
/*-----------------------------------------------------------------------*/

EXTERN int Estia_Parser(Bdd_String f);


/**AutomaticEnd***********************************************************/

#endif  /* _ESTIA */

