/**CFile*******************************************************************
  PackageName [estia]
  Synopsis    [Epistemic reasoning]

  FileName    [estiaMain.h]
  Revision    [$Revision: 2 $]
  Date        [$Date: 2006-10-05 15:30:29 +0200 (Thu, 05 Oct 2006) $]
  Authors     [Robert Meolic (meolic@uni-mb.si), 
	       Marina Bagic (marina.bagic@fer.hr)]
  Description [File estiaMain.c contains main functions.]
  SeeAlso     [estia.h, estiaInt.h]

  Copyright   [see estia.h]
  ************************************************************************/

#include "estiaInt.h"

/*-----------------------------------------------------------------------*/
/* Variable declarations                                                 */
/*-----------------------------------------------------------------------*/

int estia_status = 0;                 /* initialization status */
int estia_agents;                     /* number of agents */
int estia_compositions;               /* number of compositions */
int estia_sorts;                      /* number of sorts */
Estia_Agent *estia_agentTable;        /* table of agents */


/**AutomaticStart*********************************************************/

/*-----------------------------------------------------------------------*/
/* Static function prototypes                                            */
/*-----------------------------------------------------------------------*/

/**AutomaticEnd***********************************************************/

/*-----------------------------------------------------------------------*/
/* Definition of exported functions                                      */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    [Function Estia_Init.]
  Description [Creates tcl commands.]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

#ifndef TCLOUT
   void Estia_Init(void) {}
#endif

/**Function****************************************************************
  Synopsis    [Function Estia_InitPkg.]
  Description [Initializes the package.]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
Estia_InitPkg()
{
  if (estia_status == 1) {
    printf("\nESTIA package is already initialized. ");
    return;
  } else {
    estia_status = 1;
  }

  estia_agents = 0;                     /* number of agents */

  estia_compositions = 0;               /* number of compositions */

  estia_sorts = 0;                      /* number of sorts */


}

/**Function****************************************************************
  Synopsis    [Function Estia_ExitPkg.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
Estia_ExitPkg()
{
}

/**Function****************************************************************
  Synopsis    [Function Estia_AboutPkg]
  Description [Estia_AboutPkg reports release of Estia package.]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
Estia_AboutPkg() {
  printf("EST ");
  printf(EDITION);
  printf(", version ");
  printf(VERSION);
  printf(", ");
  printf("ESTIA package");
  printf("\n");
}



/**Function****************************************************************
  Synopsis    [int Estia_AddNewAgent(Bdd_String name)]
  Description [adds new agent to agentTable]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

int
Estia_AddNewAgent(Bdd_String name)
{
  Estia_Agent *p;
  int i;
  int agentNum;
  Bdd_Boolean OK;

  i = 0;
  if (estia_agents > 0) {
    OK = FALSE;
    while (!OK && i<estia_agents) {
      if (!strcmp(name,estia_agentTable[i].name)) {
        OK = TRUE;
      } else {
        i++;
      }
    }
  }

  if (estia_agents == 0) {
    estia_agentTable = (Estia_Agent *) malloc (sizeof(Estia_Agent));
    p = estia_agentTable;
    estia_agents = 1;
    agentNum = 0;
  } else {

    if (OK) {
      agentNum = i;
      Estia_DeleteAgent(i);
      p = &estia_agentTable[i];
      free(p->name);
      printf(" (process overwritten) ");
    } else {
      agentNum = estia_agents;
      estia_agents++;
      estia_agentTable = (Estia_Agent *) realloc (estia_agentTable, estia_agents * sizeof(Estia_Agent));
      p = &estia_agentTable[estia_agents-1];
    }
  }

  p->name = strdup(name);
  p->deleted = FALSE;
  p->sort = -1;
  p->numStates = 0;
  p->tableS = NULL;
  p->initial = -1;
  p->numTransitions = 0;
  p->tableT = NULL;
  p->tableK = NULL;
  p->numK = 0;

  return agentNum;
}

/**Function****************************************************************
  Synopsis    [void Estia_DeleteAgent(int num)]
  Description []
  SideEffects [An agent is erased and not removed from the table!
               The sequence numbers of other processes are not changed.]
  SeeAlso     []
  ************************************************************************/

void
Estia_DeleteAgent(int num)
{
  int i;
  Estia_Agent *p;

  p = &estia_agentTable[num];

  if (p->name) free(p->name);
  p->name = strdup("DELETED");

  for(i=0; i<p->numStates; i++) {
    if (p->tableS[i].name) {
      free(p->tableS[i].name);
      p->tableS[i].name = NULL;
    }
  }

  free(p->tableS);
  free(p->tableT);

  p->deleted = TRUE;
  p->sort = -1;
  p->numStates = 0;
  p->tableS = NULL;
  p->initial = -1;
  p->numTransitions = 0;
  p->tableT = NULL;
  p->numK = 0;
  p->tableK = NULL;
}

/**Function****************************************************************
  Synopsis    [void Estia_DeleteStateAgent(Pa_Process *proc, int num)]
  Description []
  SideEffects [ The state is only erased and not deleted from the table.
                Positions of states in the state table are not changed.]
  SeeAlso     []
  ************************************************************************/

void
Estia_DeleteStateAgent(Estia_Agent *agent, int num)
{
  Estia_AgentState *s;

  s = &agent->tableS[num];

  if (s->name) {
    free(s->name);
    s->name = strdup("");
  }
  s->numK = 0;
  s->tableK = NULL;
}


/**Function****************************************************************
  Synopsis    [int Estia_FOANextStateAgent(Estia_Agent *agent, Bdd_String prefix)]
  Description [Add a state to the proces. The name of the state will be
               composed of given prefix and a number.]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

int
Estia_FOANextStateAgent(Estia_Agent *agent, Bdd_String prefix)
{
  int i;
  Bdd_String x;

  if (!prefix) {
    printf("\nError: Estia_FOANextStateAgent got null string!\n");
    return -1;
  }

  x = (Bdd_String) malloc(127);
  sprintf(x,"%s%d<%s>",prefix,agent->numStates+1,agent->name);

  i = Estia_FOAStateAgent(agent,x);

  free(x);
  return i;
}


/**Function****************************************************************
  Synopsis    [int Estia_FOAStateAgent(Estia_Agent *agent, Bdd_String name)]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

int
Estia_FOAStateAgent(Estia_Agent *agent, Bdd_String name)
{
  Bdd_String name1;
  Estia_AgentState *s;
  int i;

  if (!agent) {
    printf("\nError: Estia_FOAStateAgent got null agent!\n");
    return -1;
  }

  if (!name) {
    printf("\nError: Estia_FOAStateAgent got null name!\n");
    return -1;
  }

  if (!strcmp(name,"")) {
     name1 = strdup(name);
  } else {
    if (strchr(name,'<')) {
      name1 = strdup(name);
    } else {
      name1 = (Bdd_String) malloc(127);
      sprintf(name1,"%s<%s>",name,agent->name);
    }
  }

  i = 0;
  if (agent->numStates == 0) {
    agent->numStates = 1;
    agent->tableS = (Estia_AgentState *) malloc (sizeof(Estia_AgentState));
    s = agent->tableS;
  } else {
    while (i<agent->numStates) {
      if (!strcmp(name1,agent->tableS[i].name)) {
        free (name1);
        return i;
      } else {
        i++;
      }
    }


    (agent->numStates)++;
    agent->tableS = (Estia_AgentState *) realloc (agent->tableS, agent->numStates * sizeof(Estia_AgentState));
    s = &agent->tableS[agent->numStates-1];
  }

  s->name = strdup(name1);

  free (name1);
  return (agent->numStates-1);
}


/**Function****************************************************************
  Synopsis    [int Estia_FOAGateAgent(Estia_Agent *agent, Bdd_String name)]
  Description []
  SideEffects [Almost copy of FOAState.]
  SeeAlso     []
  ************************************************************************/

int
Estia_FOAGateAgent(Estia_Agent *agent, Bdd_String name)
{
  Bdd_String name1;
  Estia_AgentState *s;
  int i = 0;

  if (!name) {
    printf("\nError: Pa_FOAGateProcess got null string!\n");
    return -1;
  }

  if (strchr(name,'<')) {
    printf("\nError: The name of GATE must not contain '<>'!\n");
    return -1;
  } else {
    name1 = strdup(name);
  }

  if (!agent) return -1;
  if (agent->numStates == 0) {
    agent->numStates = 1;
    agent->tableS = (Estia_AgentState *) malloc (sizeof(Estia_AgentState));
    s = agent->tableS;
  } else {
    while (i<agent->numStates) {
      if (!strcmp(name1,agent->tableS[i].name)) {
        free (name1);
        return i;
      } else {
        i++;
      }
    }

    (agent->numStates)++;
    agent->tableS = (Estia_AgentState *) realloc (agent->tableS, agent->numStates * sizeof(Estia_AgentState));
    s = &agent->tableS[agent->numStates-1];
  }    

  s->name = strdup(name1);

  free (name1);
  return (agent->numStates-1);
}

/*
void 
getFromKnowledge(Estia_Knowledge *knowledge, Estia_Atom atom)
{
	if(knowledge==NULL) return ("NULL pointer, no knowledge specified!\n")
	while(knowledge->numAtom){
		if(knowledge->tableA)

}
*/


/* operate on inform performative */
/* int
Estia_FOAKnowledgeAtom(Estia_Knowledge *knowledge, Bdd_String atom); 	
{
  Estia_Atom *a;
  int i;
  if(!knowledge) {
    printf ("NULL pointer, no knowledge specified!\n");
    return -1;
  }
  if(!atom) {
    printf ("NULL string, no atom specified!\n");
    return -1;
  }

  i = 1;
  while (i<knowledge->numAtoms) {
    if (!strcmp(atom,knowledge->tableA[i].name))
      return i;
    else
      i++;
  }

  (knowledge->numAtoms)++;
  if (!(knowledge->tableK = (Estia_Atom *) realloc(
	knowledge->tableK, knowledge->numAtoms * sizeof(Estia_Atom)))) {
    printf("\nError: Estia_FOAKnowledgeAtom out of memory!\n");
    return -1;
  }


  a = &knowledge->tableA[knowledge->numAtoms-1];

  a->name = strdup(atom);

  return (knowledge->numAtoms-1);
}

*/

/* void addToKnowledge();	*/	/* operate on inform-result performative*/

/* void deleteFromKnowledge(); */	/* operate on untell performative*/

/* void updateKnowledge();	*/	/* operate on update performative*/




/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

int main(int argc, char **argv)
{


  /* *********************************************************************** */
  /* *********************************************************************** */

  Estia_AboutPkg();

  /* *********************************************************************** */
  /* *********************************************************************** */

  Bdd_InitPkg();
  Pa_InitPkg();
  Versis_InitPkg();
  Estia_InitPkg();

  /* *********************************************************************** */
  /* *********************************************************************** */

}
