/**CFile*******************************************************************
  PackageName [estia]
  Synopsis    [Epistemic reasoning]

  FileName    [estiaParser.y]
  Revision    [$Revision: 2 $]
  Date        [$Date: 2006-10-05 15:30:29 +0200 (Thu, 05 Oct 2006) $]
  Authors     [Robert Meolic (meolic@uni-mb.si),
               Marina Bagic (marina.bagic@fer.hr)]
  Description [File estiaParser.y is a bison file for ESTIA parser.]
  SeeAlso     [estia.h, estiaInt.h, estiaParser.l]

  Copyright   [see estia.h]
  ************************************************************************/

%{
#include "estiaInt.h"
#define YYERROR_VERBOSE 
%}
	
%{
   typedef struct {
     int act;
     int type;
   } typedAction;
   typedef struct {
     typedAction *set;
     int len;
   } actionTrace;
%}

%{
   static void outline(char *s);
   static void lowercase(char *s);
   static void addToSet(int **set, int *len, int state);
   static void addToTrace(typedAction **set, int *len, int act, int type);
   static void addToTraceSet(actionTrace **set, int *len, actionTrace trace);
   static void writeSet(int *set, int len);
   static void writeTraceSet(actionTrace *set, int len);
%}

%{
   /* UNEXPECTED SIDEEFFECT ON SOME SYSTEMS */
   /* THE VARIABLES DEFINED HERE ARE NOT ALWAYS LOCAL, THEY CAN BE GLOBAL! */
   /* THEREFORE, I USE A PREFIX FOR EACH VARIABLE */

   int ESTIASort;
   int ESTIAProc;
   Bdd_String ESTIAinputfile;
   Bdd_String ESTIAsortname;
   Bdd_String ESTIALowerProc;
   Bdd_String ESTIAPlist;
   Bdd_String ESTIAAlist;
   Bdd_String ESTIAOld,ESTIANew;
   int ESTIANum,ESTIANump,ESTIANuma,ESTIACompType;
%}

%union {
  Bdd_String outline;
  Bdd_String word;
  Bdd_String line;
  struct {
    int *set;
    int len;
  } setOfStates;
  actionTrace traceOfActions;
  struct {
    actionTrace *set;
    int len;
  } setOfTraces;
}


%token UNKNOWN
%token <outline> OUTLINE
%token <word> PROCESS NAME ACTION SEP NET INTERLEAVE SYNCHRONOUS COMPOSE STARTPARTIAL ENDPARTIAL
%type <setOfStates> procbody
%type <traceOfActions> onetrace
%type <setOfTraces> tracelist

%left '+'
%left SEP

%%
  start		: body ;

  body          : outline body {}
                | process body {}
                | PROCESS process body {}
                | NET net body {}
                | ;

  outline       : OUTLINE {

                    Bdd_String tmp;

                    tmp = strdup($1);
                    outline(tmp);
                    free(tmp);
                    free($1);
                  };

  process	: NAME {  

                    Bdd_String name;

                    /*
                    fprintf(stderr,"NAME %s\n",$1);
		    */

                    name = strdup($1);
                    printf("  Process %s ... ",name);

                    ESTIAProc = Pa_AddNewProcess(name);
                    ESTIALowerProc = strdup(name);
                    lowercase(ESTIALowerProc);

		    ESTIAsortname = NULL;

                    free(name);
		  }
                  sort {
		    if (!ESTIAsortname) ESTIAsortname = strdup(ESTIAinputfile);
                    ESTIASort = Pa_FindSort(ESTIAsortname);
                    if (ESTIASort == -1) {
                      ESTIASort = Pa_AddNewSort(ESTIAsortname);
		    }
                    pa_processTable[ESTIAProc].sort = ESTIASort;
                    free(ESTIAsortname);
                  }
                  '='
                  procbody {

		    int i,j;
                    int state;
                    Bdd_String tmp;
                    Bdd_Boolean OK;

                    OK = TRUE;

 		    if ($6.len == 1) {
                      pa_processTable[ESTIAProc].initial = $6.set[0];
		    } else {

                      for (i=0; i<$6.len; i++) {
                        tmp = strdup(pa_processTable[ESTIAProc].tableS[$6.set[i]].name);
                        if (!strchr(tmp,'<')) {
                          OK = FALSE;
		        }
                        free(tmp);
		      }

                      if (!OK) {
                        printf("\n  ERROR in ESTIA parser: the given usage of operator '+' is not supported, yet!\n");
		      }

                      for (i=0; i<$6.len; i++) {
                        Pa_DeleteStateProcess(&pa_processTable[ESTIAProc],$6.set[i]);
		      }

                      state = Pa_FOANextStateProcess(&pa_processTable[ESTIAProc],ESTIALowerProc);
                      for (i=0; i<$6.len; i++) {
                        for(j=0; j<pa_processTable[ESTIAProc].numTransitions; j++) {
                          if (pa_processTable[ESTIAProc].tableT[j].source == $6.set[i]) {
		            pa_processTable[ESTIAProc].tableT[j].source = state;
		          }
		        }
		      }
                      pa_processTable[ESTIAProc].initial = state;
		    }

                    if (OK) {
                      printf("OK\n");
		    }

                    free($1);
                    free($6.set);
                    free(ESTIALowerProc);
                  };

  net           : NAME {

                    Bdd_String name;

		    /*
                    fprintf(stderr,"NET NAME %s\n",$1);
		    */

                    name = strdup($1);
                    printf("  Net %s\n",name);
                    printf("    Parallel composition ... ");

                    ESTIANump = 0;
                    ESTIANuma = 0;
                    ESTIAPlist = NULL;
                    ESTIAAlist = NULL;

		    ESTIAsortname = NULL;

                    free(name);
		  }

                  sort {
		    if (!ESTIAsortname) ESTIAsortname = strdup(ESTIAinputfile);
                    ESTIASort = Pa_FindSort(ESTIAsortname);
                    if (ESTIASort == -1) {
                      ESTIASort = Pa_AddNewSort(ESTIAsortname);
		    }
                    pa_processTable[ESTIAProc].sort = ESTIASort;
                    free(ESTIAsortname);
                  }

                  '=' compbody {

                    Bdd_String comp;

                    comp = strdup($1);

                    if (!pa_sortTable[ESTIASort].encoded) {
                      Pa_EncodeSort(pa_sortTable[ESTIASort].name);
		    }
                    Pa_EncodeMultiProcesses(ESTIAPlist);

                    Versis_Compose(comp,ESTIACompType,ESTIAPlist,ESTIAAlist);

                    printf("OK\n");
                    printf("    Creating process %s ... ",comp);
                    Pa_Composition2Process(comp,TRUE);
                    printf("OK\n");

		    /*
                    printf("\n");
                    Pa_WriteProcessESTIA(comp,0);
		    */

                    free(comp);
                    free(ESTIAPlist);
                    free(ESTIAAlist);
                    free($1);
                  };

  sort          : '(' NAME ')'{
                    ESTIAsortname = strdup($2);
                  }
 
                | ;


  procbody      : NAME {

                    int state;
                    Bdd_String name;

                    /*
                    fprintf(stderr,"NAME == %s\n",$1);
		    */

                    name = strdup($1);
                    state = Pa_FOAGateProcess(&pa_processTable[ESTIAProc],name);

                    $$.set = NULL;
                    $$.len = 0;
                    addToSet(&$$.set,&$$.len,state);

                    free(name);
                    free($1);
		  }

                | procbody '+' procbody {

		    int i;

		    /*
                    fprintf(stderr,"PLUS\n");
		    */

                    $$.set = NULL;
                    $$.len = 0;
                    for (i=0; i<$1.len; i++) {
                      addToSet(&$$.set,&$$.len,($1.set)[i]);
		    }
                    for (i=0; i<$3.len; i++) {
                      addToSet(&$$.set,&$$.len,($3.set)[i]);
		    }

                    free($1.set);
                    free($3.set);
                  }

                | '(' procbody ')' {

		    int i,j;
                    int state;
                    Bdd_String tmp;
                    Bdd_Boolean OK;

                    OK = TRUE;

 		    if ($2.len == 1) {
                      $$.set = $2.set;
                      $$.len = $2.len;
		    } else {

                      for (i=0; i<$2.len; i++) {
                        tmp = strdup(pa_processTable[ESTIAProc].tableS[$2.set[i]].name);
                        if (!strchr(tmp,'<')) {
                          OK = FALSE;
		        }
                        free(tmp);
		      }

                      if (!OK) {
                        printf("\n  ERROR in ESTIA parser: the given usage of operator '+' is not supported, yet!\n");
		      }

                      for (i=0; i<$2.len; i++) {
                        Pa_DeleteStateProcess(&pa_processTable[ESTIAProc],$2.set[i]);
		      }

                      state = Pa_FOANextStateProcess(&pa_processTable[ESTIAProc],ESTIALowerProc);
                      for (i=0; i<$2.len; i++) {
                        for(j=0; j<pa_processTable[ESTIAProc].numTransitions; j++) {
                          if (pa_processTable[ESTIAProc].tableT[j].source == $2.set[i]) {
		            pa_processTable[ESTIAProc].tableT[j].source = state;
		          }
		        }
		      }
                      $$.set = NULL;
                      $$.len = 0;
                      addToSet(&$$.set,&$$.len,state);
                      free($2.set);
		    }

                  }

                | ACTION SEP procbody {

		    int i;
		    int state,len,act,type;
                    Bdd_String action;

		    /*
                    fprintf(stderr,"ACTION %s\n",$1);
                    */

                    type = 0;
                    action = strdup($1);

                    if ((action[0] == '?') || (action[0] == '!')) {

                      /* action with prefix */
                      if (action[0] == '?') {
                        type = 0;
		      }
                      if (action[0] == '!') {
                        type = 1;
		      }
                      act = Pa_FOASortAction(&pa_sortTable[ESTIASort],&action[1]);

		    } else {

                      /* action with suffix */
                      len = strlen(action);
                      if (action[len-1] == '?') {
                        type = 0;
                        action[len-1]='\0';
                      } else if (action[len-1] == '!') {
                        type = 1;
                        action[len-1]='\0';
                      }
                      act = Pa_FOASortAction(&pa_sortTable[ESTIASort],action);
		    }

                    if ($3.len != 1) {
                      printf("ERROR in ESTIA parser: problem with operator '+'\n"); 
		    }

                    state = Pa_FOANextStateProcess(&pa_processTable[ESTIAProc],ESTIALowerProc);
                    Pa_FOATransition(&pa_processTable[ESTIAProc],state,act,type,($3.set)[0]);

                    $$.set = NULL;
                    $$.len = 0;
                    addToSet(&$$.set,&$$.len,state);

		    /*
                    writeSet($$.set,$$.len);
         	    */

                    free($3.set);
                    free(action);
                    free($1);
		  }

                | '(' tracelist ')' SEP procbody {

		    int i,j;
                    int act,type,state1,state2;
                    int *set;
                    int len;

                    if ($5.len != 1) {
                      printf("\n  ERROR in ESTIA parser: the given usage of operator '+' is not supported, yet!\n");
                    }

                    set=NULL;
                    len=0;
                    for (i=0; i<$2.len; i++) {

                      state2 = $5.set[0];
                      for (j=0; j<$2.set[i].len; j++) {

                        act = $2.set[i].set[j].act;
                        type = $2.set[i].set[j].type;

		        /* DEBUGING */
		        /*
                        printf("%s\n",pa_sortTable[ESTIASort].table[act].name);
		        */

                        state1 = Pa_FOANextStateProcess(&pa_processTable[ESTIAProc],ESTIALowerProc);
                        Pa_FOATransition(&pa_processTable[ESTIAProc],state1,act,type,state2);
                        state2 = state1;
                      }

                      addToSet(&set,&len,state1);
                      free($2.set[i].set);
		    }

                    for (i=0; i<len; i++) {
                      Pa_DeleteStateProcess(&pa_processTable[ESTIAProc],set[i]);
                    }

                    state1 = Pa_FOANextStateProcess(&pa_processTable[ESTIAProc],ESTIALowerProc);
                    for (i=0; i<len; i++) {
                      for(j=0; j<pa_processTable[ESTIAProc].numTransitions; j++) {
                        if (pa_processTable[ESTIAProc].tableT[j].source == set[i]) {
		          pa_processTable[ESTIAProc].tableT[j].source = state1;
	                }
	              }
	            }

                    $$.set = NULL;
                    $$.len = 0;
                    addToSet(&$$.set,&$$.len,state1);

                    free($2.set);

	 	  } ;

  tracelist     : onetrace '+' tracelist {

                    $$.set = $3.set;
                    $$.len = $3.len;
                    addToTraceSet(&$$.set,&$$.len,$1);

                  }

                | onetrace {

                    $$.set = NULL;
                    $$.len = 0;
                    addToTraceSet(&$$.set,&$$.len,$1);

                 };

  onetrace      : ACTION SEP onetrace {

		    int state,len,act,type;
                    Bdd_String action;

                    type = 0;
                    action = strdup($1);

                    if ((action[0] == '?') || (action[0] == '!')) {

                      /* action with prefix */
                      if (action[0] == '?') {
                        type = 0;
		      }
                      if (action[0] == '!') {
                        type = 1;
		      }
                      act = Pa_FOASortAction(&pa_sortTable[ESTIASort],&action[1]);

		    } else {

                      /* action with suffix */
                      len = strlen(action);
                      if (action[len-1] == '?') {
                        type = 0;
                        action[len-1]='\0';
                      } else if (action[len-1] == '!') {
                        type = 1;
                        action[len-1]='\0';
                      }
                      act = Pa_FOASortAction(&pa_sortTable[ESTIASort],action);
		    }

                    $$.set = $3.set;
                    $$.len = $3.len;
                    addToTrace(&$$.set,&$$.len,act,type);

                    free(action);
                  }

                | ACTION {

		    int state,len,act,type;
                    Bdd_String action;

                    type = 0;
                    action = strdup($1);

                    if ((action[0] == '?') || (action[0] == '!')) {

                      /* action with prefix */
                      if (action[0] == '?') {
                        type = 0;
		      }
                      if (action[0] == '!') {
                        type = 1;
		      }
                      act = Pa_FOASortAction(&pa_sortTable[ESTIASort],&action[1]);

		    } else {

                      /* action with suffix */
                      len = strlen(action);
                      if (action[len-1] == '?') {
                        type = 0;
                        action[len-1]='\0';
                      } else if (action[len-1] == '!') {
                        type = 1;
                        action[len-1]='\0';
                      }
                      act = Pa_FOASortAction(&pa_sortTable[ESTIASort],action);
		    }

                    $$.set = NULL;
                    $$.len = 0;
                    addToTrace(&$$.set,&$$.len,act,type);

                    free(action);
                  } ;

  compbody      : COMPOSE '(' proclist ')' restrlist {

                    /* actions not in restrlist go alone and synchronised */
                    /* actions in restrlist go synchronised, only */
                    /* action tau goes alone */

                    /*
		    fprintf(stderr,"COMPOSE:\n");
		    fprintf(stderr,"(nump = %d) %s\n",ESTIANump,ESTIAPlist);
                    */

                    ESTIACompType = 0;
		  }


                 | '(' oneproc COMPOSE composein ')' restrlist {

                    /* actions not in restrlist go alone and synchronised */
                    /* actions in restrlist go synchronised, only */
                    /* action tau goes alone */

                    /*
		    fprintf(stderr,"COMPOSE:\n");
		    fprintf(stderr,"(nump = %d) %s\n",ESTIANump,ESTIAPlist);
                    */

                    ESTIACompType = 0;
		  }

                | oneproc STARTPARTIAL actionlist ENDPARTIAL oneproc { 

                    /* actions not in actionlist go alone, only */
                    /* actions in actionlist go synchronised, only */
                    /* action tau goes alone */

                    /*
		    fprintf(stderr,"PARTIAL COMPOSITION:\n");
		    fprintf(stderr,"(nump = %d) %s\n",ESTIANump,ESTIAPlist);
		    fprintf(stderr,"(nump = %d) %s\n",ESTIANump,ESTIAAlist);
                    */

                    ESTIACompType = 2;
		  } ;

                | INTERLEAVE '(' proclist ')' {

                    /* all actions go alone, only */

                    /*
		    fprintf(stderr,"INTERLEAVE:\n");
		    fprintf(stderr,"(nump = %d) %s\n",ESTIANump,ESTIAPlist);
                    */

                    ESTIACompType = 2;

                  }

                | '(' oneproc INTERLEAVE interleavein ')' {

                    /* all actions go alone, only */

                    /*
		    fprintf(stderr,"INTERLEAVE:\n");
		    fprintf(stderr,"(nump = %d) %s\n",ESTIANump,ESTIAPlist);
                    */

                    ESTIACompType = 2;

                  }

                | SYNCHRONOUS '(' proclist ')' {

                    /* action tau goes alone */
                    /* all other actions go synchronised, only */

                    /*
		    fprintf(stderr,"SYNCHRONOUS:\n");
		    fprintf(stderr,"(nump = %d) %s\n",ESTIANump,ESTIAPlist);
                    */

                    ESTIACompType = 1;

		  }

                | '(' oneproc SYNCHRONOUS synchronousin ')' {

                    /* action tau goes alone */
                    /* all other actions go synchronised, only */

                    /*
		    fprintf(stderr,"SYNCHRONOUS:\n");
		    fprintf(stderr,"(nump = %d) %s\n",ESTIANump,ESTIAPlist);
                    */

                    ESTIACompType = 1;

		  } ;

  composein     : oneproc COMPOSE composein 
                | oneproc ;

  interleavein  : oneproc INTERLEAVE interleavein
                | oneproc ;

  synchronousin : oneproc SYNCHRONOUS synchronousin
                | oneproc ;

  actionlist	: oneaction ',' moreactions 
                | oneaction
                | ;

  moreactions	: oneaction ',' moreactions 
                | oneaction ;

  oneaction    : NAME {
                    Bdd_String name;
                    Bdd_String tmp;

                    /*
                    fprintf(stderr,"NAME %s\n",$1);
		    */

                    name = strdup($1);
                    Pa_FOASortAction(&pa_sortTable[ESTIASort],name);

                    ESTIANuma++;
                    if (!ESTIAAlist) {
                      ESTIAAlist = strdup(name);
		    } else {
                      tmp = (Bdd_String) malloc(1024);
                      sprintf(tmp,"%s %s",ESTIAAlist,name);
                      free(ESTIAAlist);
                      ESTIAAlist = strdup(tmp);
                      free(tmp);
		    }

                    free(name);
                    free($1);
                  } ;

  proclist	: oneproc ',' proclist 
                | oneproc ;

  oneproc       : NAME {

                    /*
                    fprintf(stderr,"NAME %s\n",$1);
		    */

                    ESTIANum = 0;
                    ESTIAOld = NULL;
                    ESTIANew = NULL;

                  } renlist {

                    Bdd_String name,newproc;
                    Bdd_String tmp;
                    int i;

                    name = strdup($1);

                    i = 0;
                    newproc = NULL;
                    do {
                      i++;
                      if (newproc) free(newproc);
                      newproc = (Bdd_String) malloc(127);
                      sprintf(newproc,"%s%d",name,i);
                    } while (Pa_FindProcess(newproc) != -1);

                    ESTIANump++;
                    if (!ESTIAPlist) {
                      ESTIAPlist = strdup(newproc);
		    } else {
                      tmp = (Bdd_String) malloc(1024);
                      sprintf(tmp,"%s %s",ESTIAPlist,newproc);
                      free(ESTIAPlist);
                      ESTIAPlist = strdup(tmp);
                      free(tmp);
		    }

                    /*
                    fprintf(stderr,"Renaming old = [%s]\n",ESTIAOld);
                    fprintf(stderr,"Renaming new = [%s]\n",ESTIANew);
		    */

                    Pa_CopyProcess(name,newproc,pa_sortTable[ESTIASort].name,ESTIAOld,ESTIANew);

                    free(name);
                    free(newproc);
                    free(ESTIAOld);
                    free(ESTIANew);
                    free($1);
                  };

  renlist       : renaction renlist
                | ;

  renaction     : '['
                  NAME
                  '/'
                  NAME
                  ']' {

                    Bdd_String name1,name2;
                    Bdd_String tmp;

                    name1 = strdup($2);
                    name2 = strdup($4);

                    ESTIANum++;

                    Pa_FOASortAction(&pa_sortTable[ESTIASort],name1);
                    if (!ESTIANew) {
                      ESTIANew = strdup(name1);
		    } else {
                      tmp = (Bdd_String) malloc(1024);
                      sprintf(tmp,"%s %s",ESTIANew,name1);
                      free(ESTIANew);
                      ESTIANew = strdup(tmp);
                      free(tmp);
		    }

                    Pa_FOASortAction(&pa_sortTable[ESTIASort],name2);
                    if (!ESTIAOld) {
                      ESTIAOld = strdup(name2);
		    } else {
                      tmp = (Bdd_String) malloc(1024);
                      sprintf(tmp,"%s %s",ESTIAOld,name2);
                      free(ESTIAOld);
                      ESTIAOld = strdup(tmp);
                      free(tmp);
		    }

                    free(name1);
                    free(name2);

                    free($2);
                    free($4);
                  } ;

  restrlist     : restraction restrlist
                | ;

  restraction   : '\\' NAME {

                    Bdd_String name;
                    Bdd_String tmp;

                    /*
                    fprintf(stderr,"NAME %s\n",$2);
		    */

                    name = strdup($2);
                    Pa_FOASortAction(&pa_sortTable[ESTIASort],name);

                    ESTIANuma++;
                    if (!ESTIAAlist) {
                      ESTIAAlist = strdup(name);
		    } else {
                      tmp = (Bdd_String) malloc(1024);
                      sprintf(tmp,"%s %s",ESTIAAlist,name);
                      free(ESTIAAlist);
                      ESTIAAlist = strdup(tmp);
                      free(tmp);
		    }

                    free(name);
                    free($2);
                  };

%%

#include <lex.estia.c>

/*-----------------------------------------------------------------------*/
/* Definition of exported functions                                      */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

int
Estia_Parser(Bdd_String f)
{
  YY_BUFFER_STATE buffer;

  printf("\n");
  yyin = fopen(f,"r");
  if (yyin != NULL) {
    ESTIAinputfile=strdup(f);

    buffer = yy_create_buffer(yyin,YY_BUF_SIZE);
    yy_switch_to_buffer(buffer);
    yyparse();
    yy_delete_buffer(buffer);

    fclose(yyin);
  } else {
    printf("\n  ERROR: I can not open file %s.\n",f);
  }
  return 0;
}

/*-----------------------------------------------------------------------*/
/* Definition of static functions                                        */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static void
outline(char *s)
{
  if (strcmp(s,"")) printf(&s[1]);
  printf("\n");
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static void
lowercase(char *s)
{
   int  i = 0;

   while ( s[i] )
   {
      s[i] = tolower(s[i]);
      i++;
   }

   return;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static void
addToSet(int **set, int *len, int state)
{
  if (!*set) {
    *set = (int *) malloc (sizeof(int));
    (*set)[0] = state;
    *len = 1;
  } else {
    (*len)++;
    *set = (int *) realloc (*set, (*len) * sizeof(int));
    (*set)[(*len) - 1] = state;
  }
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static void
addToTrace(typedAction **set, int *len, int act, int type)
{
  if (!*set) {
    *set = (typedAction *) malloc (sizeof(typedAction));
    (*set)[0].act = act;
    (*set)[0].type = type;
    *len = 1;
  } else {
    (*len)++;
    *set = (typedAction *) realloc (*set, (*len) * sizeof(typedAction));
    (*set)[(*len) - 1].act = act;
    (*set)[(*len) - 1].type = type;
  }
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static void
addToTraceSet(actionTrace **set, int *len, actionTrace trace)
{
  if (!*set) {
    *set = (actionTrace *) malloc (sizeof(actionTrace));
    (*set)[0] = trace;
    *len = 1;
  } else {
    (*len)++;
    *set = (actionTrace *) realloc (*set, (*len) * sizeof(actionTrace));
    (*set)[(*len) - 1] = trace;
  }
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static void
writeSet(int *set, int len)
{
  int i;

  fprintf(stderr,"SET:");
  for (i=0; i<len; i++) {
    fprintf(stderr," %d",set[i]);
  }
  fprintf(stderr,"\n");
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static void
writeTraceSet(actionTrace *set, int len)
{
  int i,j;

  fprintf(stderr,"TRACE SET:\n");
  for (i=0; i<len; i++) {
    fprintf(stderr,"+\n");
    for (j=0; j<set[i].len; j++) {
      fprintf(stderr," act=%d, type=%d\n",set[i].set[j].act,set[i].set[j].type);
    }
  }
  fprintf(stderr,"\n");
}

/*-----------------------------------------------------------------------*/
/* Definition of parser internal functions                               */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

int
yyerror(char *s)
{
  printf("\n  ERROR: ");
  if (strstr(s,"parse error, unexpected UNKNOWN")) {
     printf("unknown or badly placed symbol '%s'",yytext);
  } else
  if (strcmp(s,"parse error, unexpected UNKNOWN, expecting ACTION")) {
     printf("unknown or badly placed symbol '%s'",yytext);
  } else
  if (!strcmp(s,"parse error, unexpected NAME, expecting ACTION")) {
     printf("incorrect action name '%s', missing ! or ?",yytext);
  } else
  if (!strcmp(s,"parse error, unexpected ACTION, expecting NAME")) {
     printf("incorrect process name '%s'",yytext);
  } else {
    printf("%s",s); 
  }
  printf("\n");
  return 1;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

int
yywrap()
{
  return 1;
}
