/**CFile*******************************************************************
  PackageName [estia]
  Synopsis    [Epistemic reasoning]

  FileName    [estiaTcl.h]
  Revision    [$Revision: 2 $]
  Date        [$Date: 2006-10-05 15:30:29 +0200 (Thu, 05 Oct 2006) $]
  Authors     [Robert Meolic (meolic@uni-mb.si),
               Marina Bagic (marina.bagic@fer.hr)]
  Description [File estiaTcl.c contains definitions of Tcl commands,
               which can be used for manipulating from Tcl interpreter
               (e.g. tclsh or wish).]
  SeeAlso     [estia.h, estiaInt.h]

  Copyright   [see estia.h]
  ************************************************************************/

#include "estiaInt.h"

/* on tcl 8.3 use #define USECONST */
/* on tcl 8.4 use #define USECONST const*/
/* this is defined in Makefile */

/*-----------------------------------------------------------------------*/
/* Constant declarations                                                 */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Variable declarations                                                 */
/*-----------------------------------------------------------------------*/

/**AutomaticStart*********************************************************/

/*-----------------------------------------------------------------------*/
/* Static function prototypes                                            */
/*-----------------------------------------------------------------------*/

static int EstiaInitPkgCmd(ClientData clientData, Tcl_Interp *interp,
                        int argc, USECONST char **argv);

static int EstiaAboutPkgCmd(ClientData clientData, Tcl_Interp *interp,
                         int argc, USECONST char **argv);

static int EstiaParserCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

/**AutomaticEnd***********************************************************/

/*-----------------------------------------------------------------------*/
/* Definition of exported functions                                      */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    [Function Estia_Init.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

int
Estia_Init(Tcl_Interp *interp)
{
  Tcl_CreateCommand(interp, "estia_initPkg", EstiaInitPkgCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "estia_aboutPkg", EstiaAboutPkgCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "estia_parser", EstiaParserCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  return Tcl_PkgProvide(interp, "estia", "1.0");
}

/*-----------------------------------------------------------------------*/
/* Definition of internal functions                                      */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Definition of static functions                                        */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    [Function EstiaInitCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
EstiaInitPkgCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                  USECONST char **argv)
{
  if (argc != 1) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  printf("Initialization of Estia package... ");
  Estia_InitPkg();
  printf("OK");

  printf("\n");
  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function EstiaAboutPkgCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
EstiaAboutPkgCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                   USECONST char **argv)
{
  if (argc != 1) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  Estia_AboutPkg();

  printf("\n");
  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function EstiaParserCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
EstiaParserCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                      USECONST char **argv)
{
  Bdd_String s1;

  if (argc != 2) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);

  printf("Reading file: %s",s1);
  Estia_Parser(s1);

  free(s1);

  printf("\n");
  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}
