/**CFile*******************************************************************
  PackageName [mc]
  Synopsis    [Package 'mc' provides a symbolic ACTLW model checker]

  FileName    [mc.h]
  Revision    [$Revision: 2 $]
  Date        [$Date: 2006-10-05 15:30:29 +0200 (Thu, 05 Oct 2006) $]
  Author      [Robert Meolic (meolic@uni-mb.si)]
  Description [File mc.h contains declaration of all external
               data structures needed for ACTLW model checking.]
  SeeAlso     [mcInt.h]

  Copyright   [Copyright (C) 2000-2006 UM-FERI
               This program is free software; you can redistribute it
               and/or modify it under the terms of the GNU General Public
               License (version 2) as published by the Free Software
               Foundation.

               This program is distributed in the hope that it will be useful,
               but WITHOUT ANY WARRANTY; without even the implied warranty of
               MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
               GNU General Public License for more details.

               You should have received a copy of the GNU General Public
               License along with this program; if not, write to the Free
               Software Foundation, Inc., 59 Temple Place, Suite 330,
               Boston, MA  02111-1307 USA.]
  ************************************************************************/

#ifndef _MC
#define _MC

#include <pthread.h>
#include <sched.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <stdarg.h>

#include <unistd.h>
#include <math.h>
#include <time.h>

#include <bdd.h>
#include <pa.h>
#include <versis.h>

/* ON WIN32 THERE HAS TO BE DEFINED ESTWIN32 */
/* ON UNIX THERE HAS TO BE DEFINED ESTUNIX */

#ifdef ESTUNIX
#  ifndef EXTERN
#  define EXTERN extern
#  endif
#endif

#ifdef ESTWIN32
#ifndef TCL
#    ifdef BUILD_MC
#      undef EXTERN
#      define EXTERN __declspec (dllexport)
#    else
#      ifdef USE_MC
#        undef EXTERN
#        define EXTERN __declspec (dllimport)
#      else
#        define EXTERN extern
#      endif
#    endif
#else
#    ifdef BUILD_MC
#      undef TCL_STORAGE_CLASS
#      define TCL_STORAGE_CLASS DLLEXPORT
#    else
#      undef TCL_STORAGE_CLASS
#      define TCL_STORAGE_CLASS DLLIMPORT
#    endif
#endif
#endif

/*-----------------------------------------------------------------------*/
/* Constant declarations                                                 */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Type declarations                                                     */
/*-----------------------------------------------------------------------*/

#ifndef _COMMONTYPE
#define _COMMONTYPE
typedef void *pvoid;
#endif

/*-----------------------------------------------------------------------*/
/* Structure declarations                                                */
/*-----------------------------------------------------------------------*/

typedef struct {
  Bdd_String     name;                       /* formula name */
  Bdd_String     formula;                    /* formula body */
} Mc_Formula;

/*-----------------------------------------------------------------------*/
/* Variable declarations                                                 */
/*-----------------------------------------------------------------------*/

EXTERN int mc_status;

EXTERN int mc_formulae;                      /* number of formulae */

EXTERN Mc_Formula *mc_formulaTable;          /* table of formulae */

/* ACTL PARSER SETTINGS */
EXTERN const int MC_SIMPLE;
EXTERN const int MC_TRACE;         /* trace of ACTL parsing */
EXTERN const int MC_XTRACE;        /* extended trace of ACTL parsing */
EXTERN const int MC_EXPLAIN;       /* explain generated diagnostic */
EXTERN const int MC_TRACEPATH;     /* full paths in diagnostic */
EXTERN const int MC_DIAGNOSTIC;    /* generate linear w/c */
EXTERN const int MC_AUTOMATON;     /* generate w/c automaton */
EXTERN const int MC_MINIMIZE;      /* minimize w/c automaton */
EXTERN const int MC_TREE;          /* generate tree-like w/c */

/*-----------------------------------------------------------------------*/
/* Macro declarations                                                    */
/*-----------------------------------------------------------------------*/

/**AutomaticStart*********************************************************/

/*-----------------------------------------------------------------------*/
/* Function prototypes                                                   */
/*-----------------------------------------------------------------------*/

EXTERN void Mc_InitPkg();

EXTERN void Mc_ExitPkg();

EXTERN void Mc_AboutPkg();

EXTERN int Mc_AddNewFormula(Bdd_String name, Bdd_String formula);

EXTERN int Mc_FOANextFormula(Bdd_String f, Bdd_String prefix);

EXTERN int Mc_FindFormula(Bdd_String name);

EXTERN void Mc_WriteKnownFormulae();

EXTERN void Mc_WriteFormula(Bdd_String ime, char type);

EXTERN void Mc_CheckACTL(int par, Bdd_String name, Bdd_String f,
                         int parserSettings);

EXTERN void Mc_CheckACTLString(int par, Bdd_String name, Bdd_String f,
                         int parserSettings);

EXTERN void Mc_CheckACTLFile(int par, Bdd_String name, Bdd_String f,
                         int parserSettings);

EXTERN void Mc_CheckACTL_Thr(int par, Bdd_String name, Bdd_String f,
                             int parserSettings);

EXTERN void Mc_CheckACTLString_Thr(int par, Bdd_String name, Bdd_String f,
                             int parserSettings);

EXTERN void Mc_CheckACTLFile_Thr(int par, Bdd_String name, Bdd_String f,
                         int parserSettings);

EXTERN void Mc_GenerateWCA(Bdd_String name, Bdd_String f, Bdd_String aname);

EXTERN void Mc_ReadACTL(Bdd_String name, Bdd_String f);

EXTERN void Mc_ReadACTLFile(Bdd_String f);

EXTERN Bdd_Edge Mc_CheckEEU(Bdd_Edge D, Bdd_Edge a1, Bdd_Edge s1, 
                           Bdd_Edge a2, Bdd_Edge s2);

EXTERN Bdd_Edge Mc_CheckAAU(Bdd_Edge D, Bdd_Edge a1, Bdd_Edge s1, 
                           Bdd_Edge a2, Bdd_Edge s2, Bdd_Edge *AWfun,
                           Bdd_Edge *AFfun);

EXTERN Bdd_Edge Mc_CheckEEW(Bdd_Edge D, Bdd_Edge a1, Bdd_Edge s1, 
                            Bdd_Edge a2, Bdd_Edge s2, Bdd_Edge *EUfun,
                            Bdd_Edge *EGfun);

EXTERN Bdd_Edge Mc_CheckAAW(Bdd_Edge D, Bdd_Edge a1, Bdd_Edge s1, 
                            Bdd_Edge a2, Bdd_Edge s2);

EXTERN Bdd_Edge Mc_CheckEEX(Bdd_Edge D, Bdd_Edge a, Bdd_Edge s);

EXTERN Bdd_Edge Mc_CheckAAX(Bdd_Edge D, Bdd_Edge a, Bdd_Edge s);

EXTERN Bdd_Edge Mc_CheckEEF(Bdd_Edge D, Bdd_Edge a, Bdd_Edge s);

EXTERN Bdd_Edge Mc_CheckAAF(Bdd_Edge D, Bdd_Edge a, Bdd_Edge s);

EXTERN Bdd_Edge Mc_CheckEEG(Bdd_Edge D, Bdd_Edge a, Bdd_Edge s);

EXTERN Bdd_Edge Mc_CheckAAG(Bdd_Edge D, Bdd_Edge a, Bdd_Edge s);

EXTERN Bdd_Edge Mc_CheckEU(Bdd_Edge D, Bdd_Edge s1, Bdd_Edge a1,
                           Bdd_Edge a2, Bdd_Edge s2);

EXTERN Bdd_Edge Mc_CheckAU(Bdd_Edge D, Bdd_Edge s1, Bdd_Edge a1,
                           Bdd_Edge a2, Bdd_Edge s2);

EXTERN Bdd_Edge Mc_CheckEX(Bdd_Edge D, Bdd_Edge a, Bdd_Edge s);

EXTERN Bdd_Edge Mc_CheckAX(Bdd_Edge D, Bdd_Edge a, Bdd_Edge s);

EXTERN Bdd_Edge Mc_CheckHME(Bdd_Edge D, Bdd_Edge s1, Bdd_Edge a,
                            Bdd_Edge s2);

EXTERN Bdd_Edge Mc_CheckHMA(Bdd_Edge D, Bdd_Edge a, Bdd_Edge s);

EXTERN Bdd_Edge Mc_CheckEF(Bdd_Edge D, Bdd_Edge s);

EXTERN Bdd_Edge Mc_CheckAF(Bdd_Edge D, Bdd_Edge s);

EXTERN Bdd_Edge Mc_CheckEG(Bdd_Edge D, Bdd_Edge a);

EXTERN Bdd_Edge Mc_CheckAG(Bdd_Edge D, Bdd_Edge a);

/**AutomaticEnd***********************************************************/

#endif  /* _MC */
