/**CFile*******************************************************************
  PackageName [mc]
  Synopsis    [Package 'mc' provides a symbolic ACTLW model checker]

  FileName    [mcThread.c]
  Revision    [$Revision: 2 $]
  Date        [$Date: 2006-10-05 15:30:29 +0200 (Thu, 05 Oct 2006) $]
  Author      [Robert Meolic (meolic@uni-mb.si)]
  Description [File mcThread.c contains functions needed if using
               threads.]
  SeeAlso     [mc.h, mcInt.h]

  Copyright   [see mc.h]
  ************************************************************************/

#include "mcInt.h"

/*-----------------------------------------------------------------------*/
/* Constant declarations                                                 */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Variable declarations                                                 */
/*-----------------------------------------------------------------------*/

/**AutomaticStart*********************************************************/

/*-----------------------------------------------------------------------*/
/* Static function prototypes                                            */
/*-----------------------------------------------------------------------*/

/**AutomaticEnd***********************************************************/

/*-----------------------------------------------------------------------*/
/* Definition of thread functions                                        */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
McCheckACTL_Thr(pvoid args)
{
  struct funparam {
    int par;
    Bdd_String name;
    Bdd_String f;
    int parserSettings;
    int *thread_done;
  };

  int par;
  Bdd_String name;
  Bdd_String f;
  int parserSettings;

  par = ((struct funparam *) args)->par;
  name = ((struct funparam *) args)->name;
  f = ((struct funparam *) args)->f;
  parserSettings = ((struct funparam *) args)->parserSettings;

  Mc_CheckACTL(par,name,f,parserSettings);

  *((struct funparam *) args)->thread_done = 1;

  return;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
McCheckACTLString_Thr(pvoid args)
{
  struct funparam {
    int par;
    Bdd_String name;
    Bdd_String f;
    int parserSettings;
    int *thread_done;
  };

  int par;
  Bdd_String name;
  Bdd_String f;
  int parserSettings;

  par = ((struct funparam *) args)->par;
  name = ((struct funparam *) args)->name;
  f = ((struct funparam *) args)->f;
  parserSettings = ((struct funparam *) args)->parserSettings;

  Mc_CheckACTLString(par,name,f,parserSettings);

  *((struct funparam *) args)->thread_done = 1;

  return;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
McCheckACTLFile_Thr(pvoid args)
{
  struct funparam {
    int par;
    Bdd_String name;
    Bdd_String f;
    int parserSettings;
    int *thread_done;
  };

  int par;
  Bdd_String name;
  Bdd_String f;
  int parserSettings;

  par = ((struct funparam *) args)->par;
  name = ((struct funparam *) args)->name;
  f = ((struct funparam *) args)->f;
  parserSettings = ((struct funparam *) args)->parserSettings;

  Mc_CheckACTLFile(par,name,f,parserSettings);

  *((struct funparam *) args)->thread_done = 1;

  return;
}

/*-----------------------------------------------------------------------*/
/* Definition of exported functions                                      */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
Mc_CheckACTL_Thr(int par, Bdd_String name, Bdd_String f, int parserSettings)
{
  struct {
    int par;
    Bdd_String name;
    Bdd_String f;
    int parserSettings;
    int *thread_done;
  } funparam;

  pthread_t thread;
  pthread_attr_t attr;
  struct sched_param param;

  int thread_done;

  /*
  pthread_attr_init(&attr);
  pthread_attr_setstacksize(&attr,262144);
  pthread_attr_setinheritsched(&attr,PTHREAD_EXPLICIT_SCHED);
  pthread_attr_setschedpolicy(&attr,SCHED_OTHER);
  param.sched_priority = sched_get_priority_min(SCHED_OTHER);
  pthread_attr_setschedparam(&attr, &param);
  */

  funparam.par = par;
  funparam.name = name;
  funparam.f = f;
  funparam.parserSettings = parserSettings;
  funparam.thread_done = &thread_done;

  /* POSIX */
  outflag = 0;
  outthread = FALSE;
  thread_done = 0;
  if (pthread_create(&thread,NULL,(void *)&McCheckACTL_Thr,&funparam)) {
    outthread = FALSE;
    printf("Error: pthread_create\n");
  } else {
    sched_yield();
    while (!thread_done) {
#ifdef TCLOUT
      Tcl_DoOneEvent(TCL_DONT_WAIT);
#endif
      if (outflag) {
        Mi_Out("%s",out);
        outflag = 0;
      }
    }
    if (pthread_join(thread,NULL)) {
      Mi_Out("Error: pthread_join\n");
    }
    outthread = FALSE;
  }

  return;
}


/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
Mc_CheckACTLString_Thr(int par, Bdd_String name, Bdd_String f, int parserSettings)
{
  struct {
    int par;
    Bdd_String name;
    Bdd_String f;
    int parserSettings;
    int *thread_done;
  } funparam;

  pthread_t thread;
  pthread_attr_t attr;
  struct sched_param param;

  int thread_done;

  /*
  pthread_attr_init(&attr);
  pthread_attr_setstacksize(&attr,262144);
  pthread_attr_setinheritsched(&attr,PTHREAD_EXPLICIT_SCHED);
  pthread_attr_setschedpolicy(&attr,SCHED_OTHER);
  param.sched_priority = sched_get_priority_min(SCHED_OTHER);
  pthread_attr_setschedparam(&attr, &param);
  */

  funparam.par = par;
  funparam.name = name;
  funparam.f = f;
  funparam.parserSettings = parserSettings;
  funparam.thread_done = &thread_done;

  /* POSIX */
  outflag = 0;
  outthread = FALSE;
  thread_done = 0;
  if (pthread_create(&thread,NULL,(void *)&McCheckACTLString_Thr,&funparam)) {
    outthread = FALSE;
    printf("Error: pthread_create\n");
  } else {
    sched_yield();
    while (!thread_done) {
#ifdef TCLOUT
      Tcl_DoOneEvent(TCL_DONT_WAIT);
#endif
      if (outflag) {
        Mi_Out("%s",out);
        outflag = 0;
      }
    }
    if (pthread_join(thread,NULL)) {
      Mi_Out("Error: pthread_join\n");
    }
    outthread = FALSE;
  }

  return;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
Mc_CheckACTLFile_Thr(int par, Bdd_String name, Bdd_String f, int parserSettings)
{
  struct {
    int par;
    Bdd_String name;
    Bdd_String f;
    int parserSettings;
    int *thread_done;
  } funparam;

  pthread_t thread;
  pthread_attr_t attr;
  struct sched_param param;

  int thread_done;

  /*
  pthread_attr_init(&attr);
  pthread_attr_setstacksize(&attr,262144);
  pthread_attr_setinheritsched(&attr,PTHREAD_EXPLICIT_SCHED);
  pthread_attr_setschedpolicy(&attr,SCHED_OTHER);
  param.sched_priority = sched_get_priority_min(SCHED_OTHER);
  pthread_attr_setschedparam(&attr, &param);
  */

  funparam.par = par;
  funparam.name = name;
  funparam.f = f;
  funparam.parserSettings = parserSettings;
  funparam.thread_done = &thread_done;

  /* POSIX */
  outflag = 0;
  outthread = FALSE;
  thread_done = 0;
  if (pthread_create(&thread,NULL,(void *)&McCheckACTLFile_Thr,&funparam)) {
    outthread = FALSE;
    printf("Error: pthread_create\n");
  } else {
    sched_yield();
    while (!thread_done) {
#ifdef TCLOUT
      Tcl_DoOneEvent(TCL_DONT_WAIT);
#endif
      if (outflag) {
        Mi_Out("%s",out);
        outflag = 0;
      }
    }
    if (pthread_join(thread,NULL)) {
      Mi_Out("Error: pthread_join\n");
    }
    outthread = FALSE;
  }

  return;
}
