/**CFile*******************************************************************
  PackageName [pa]
  Synopsis    [Package 'pa' is a framework for implementation of 
               process algebrae similar to Milner's CCS and Hoare's CSP.]

  FileName    [paParser.y]
  Revision    [$Revision: 2 $]
  Date        [$Date: 2006-10-05 15:30:29 +0200 (Thu, 05 Oct 2006) $]
  Authors     [Robert Meolic (meolic@uni-mb.si)]
  Description [File paParser.y is a bison file of parser for textual
               description of LTSs.]
  SeeAlso     [pa.h, paInt.h, paParser.l]

  Copyright   [see pa.h]
  ************************************************************************/

%{
#include "paInt.h"
#define YYERROR_VERBOSE 
%}

%{
   static char FOATransition(Bdd_String state1,Bdd_String act,Bdd_String state2);
   static void outline(char *s);
%}

%{
   /* UNEXPECTED SIDEEFFECT ON SOME SYSTEMS */
   /* THE VARIABLES DEFINED HERE ARE NOT ALWAYS LOCAL, THEY CAN BE GLOBAL! */
   /* THEREFORE, I USE A PREFIX FOR EACH VARIABLE */

   /* STATIC VARIABLES? I DO NOT KNOW, IF THIS IS A GOOD OR A BAD IDEA */
   /*
   static int PaSort;
   static int PaProc;
   static Bdd_String PaState1,PaState2;
   */

   int PaSort;
   int PaProc;
   Bdd_String PaState1,PaState2;
%}

%union {
  Bdd_String outline;
  Bdd_String word;
}

%token UNKNOWN
%token <outline> OUTLINE
%token <word> NAME ACTION ACTIONS GATES
%token <word> INITIAL_STATE FINAL_STATES PROCESS SORT TRANSITIONS 

%%
  start		: sorts processes ;

  sorts		: sort sorts 
                | ;

  sort	        : OUTLINE {
                    Bdd_String tmp;

                    tmp = strdup($1);
                    outline(tmp);
                    free(tmp);
                    free($1);
                  }

                | SORT NAME {

                    Bdd_String name;

                    /*
                    fprintf(stderr,"NAME %s\n",$2);
		    */

                    name = strdup($2);

                    printf("  Sort %s ... ",name);
 		    PaSort = Pa_AddNewSort(name);
                    free(name);
                    free($2);
		  }
                  elements {

                    printf("OK\n");

		  }
                | error sorts {
                    yyerrok;
                  } ;

  elements	: element ',' elements
	  	| element ;

  element	: NAME {

                    Bdd_String name,errtext;

                    name = strdup($1);
                    if (!pa_sorts) {
                      errtext = (Bdd_String) malloc(255);
                      sprintf(errtext,"'%s'",name);
                      yyerror(errtext);
                      free(errtext);
                      free(name);
                      free($1);
                      YYERROR;
		    } else {
                      Pa_FOASortAction(&pa_sortTable[PaSort],name);
                      free(name);
                      free($1);
		    }
		  } ;
 
  processes	: proces processes 
                | ;

  proces	: OUTLINE {
                    Bdd_String tmp;

                    tmp = strdup($1);
                    outline(tmp);
                    free(tmp);
                    free($1);
                  }

                | PROCESS NAME {  

                    Bdd_String name;

                    /*
                    fprintf(stderr,"NAME %s\n",$2);
		    */

                    name = strdup($2);

                    printf("  Process %s ... ",name);
                    PaProc = Pa_AddNewProcess(name);
                    free(name);
		  }
                  SORT dogsort
                  actions
                  INITIAL_STATE initial_state
                  final
                  gates
	  	  TRANSITIONS prehodi {
                    printf("OK\n");
                    free($2);
		  }
  
                | error processes {
                    yyerrok;
                  } ;

  dogsort	: NAME {

                    Bdd_String name;

                    name = strdup($1);
                    pa_processTable[PaProc].sort = Pa_FindSort(name);
                    free(name);
                    free($1);
		  } ;

  actions       :
                | ACTIONS actionsname {
                     printf("\n  Option ACTIONS is depreciated. Please, remove it! ");
		  } ;

  actionsname   :
                | ACTION {free($1);} ',' actionsname
                | ACTION {free($1);} ;

  initial_state : NAME {

                    Bdd_String name;

                    name = strdup($1);
                    pa_processTable[PaProc].initial =
                       Pa_FOAStateProcess(&pa_processTable[PaProc],name);
                    free(name);
                    free($1);
                  } ;

  final         :
                | FINAL_STATES finalnames {
		  } ;

  finalnames    :
                | NAME {

                    Bdd_String name;
                    int final;
                    int i;

                    name = strdup($1);
                    i = Pa_FOAStateProcess(&pa_processTable[PaProc],name);
                    pa_processTable[PaProc].tableS[i].final = TRUE;

		    /* MARK FINAL STATES WITH ADDITIONAL TRANSITION */
		    /*
                    final = Pa_FOAStateProcess(&pa_processTable[PaProc],"FINAL");
                    Pa_FOATransition(&pa_processTable[PaProc],
                       i,Pa_FindSortAction(&pa_sortTable[pa_processTable[PaProc].sort],"FINAL"),1,final);
		    */

                    free(name);
                    free($1);
                  } finalnames ;

  gates         :
                | GATES gatenames {
                     printf("\n  Option GATES is depreciated. Please, remove it! ");
		  } ;

  gatenames     :
                | NAME {free($1);} gatenames ;

  prehodi       : enostanje prehodi
                | ;

  enostanje	: NAME {
                    PaState1 = strdup($1);
                  }
                  '=' prehod {
                    free($1);
                  };
  
  prehod	: enprehod '+' prehod
	 	| enprehod {
                    free(PaState1);
                  } ;

  enprehod 	: ACTION '.' NAME {

                    Bdd_String akcija;
                    Bdd_String errtext;

                    akcija = strdup($1);
                    PaState2 = strdup($3);

                    if (FOATransition(PaState1,akcija,PaState2)) {
                      errtext = (Bdd_String) malloc(255);
                      sprintf(errtext,"Unknown action '%s'",akcija);
                      yyerror(errtext);
                      free(errtext);
                      free(akcija);
                      free(PaState2);
                      free($1);
                      free($3);
                      YYERROR;
		    } else {
                      free(akcija);
                      free(PaState2);
                      free($1);
                      free($3);
		    }

		  } ;

%%

#include <lex.pa.c>

/*-----------------------------------------------------------------------*/
/* Definition of exported functions                                      */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

int
Pa_ParseSortProcess(Bdd_String ime)
{
  YY_BUFFER_STATE buffer;

  printf("\n");
  yyin = fopen(ime,"r");
  if (yyin != NULL) {
    PaProc = -1;
    PaSort = -1;

    buffer = yy_create_buffer(yyin,YY_BUF_SIZE);
    yy_switch_to_buffer(buffer);
    yyparse();
    yy_delete_buffer(buffer);

    fclose(yyin);
  } else {
    printf("ERROR: Cannot open file %s.\n",ime);
  }

  return 0;
}

/*-----------------------------------------------------------------------*/
/* Definition of static functions                                        */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static char
FOATransition(Bdd_String state1, Bdd_String act, Bdd_String state2)
{
  Pa_Process *p;
  int s1,s2;
  int a;
  int type;
  int len;
  Bdd_String action;

  action = strdup(act);

  p = &pa_processTable[PaProc];
  type = 0;
  len = strlen(action);
  if (action[len-1] == '?') {
    type = 0;
    action[len-1]='\0';
  } else if (action[len-1] == '!') {
    type = 1;
    action[len-1]='\0';
  }

  a = Pa_FindSortAction(&pa_sortTable[p->sort],action);
  if (a == -1) {
    return 1;
  }

  s1 = Pa_FOAStateProcess(p,state1);
  s2 = Pa_FOAStateProcess(p,state2);

  Pa_FOATransition(p,s1,a,type,s2);

  free(action); 

  return 0;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static void
outline(char *s)
{
  if (strcmp(s,"")) printf(&s[1]);
  printf("\n");
}

/*-----------------------------------------------------------------------*/
/* Definition of parser internal functions                               */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

int
yyerror(char *s)
{
  printf("    ERROR: ");
  if (strstr(s,"parse error, unexpected UNKNOWN")) {
     printf("unknown or badly placed symbol '%s'",patext);
  } else
  if (strcmp(s,"parse error, unexpected UNKNOWN, expecting ACTION")) {
     printf("unknown or badly placed symbol '%s'",patext);
  } else
  if (!strcmp(s,"parse error, unexpected NAME, expecting ACTION")) {
     printf("incorrect action name '%s', missing ! or ?",palval);
  } else
  if (!strcmp(s,"parse error, unexpected UNKNOWN, expecting '.'")) {
     printf("incorrect action name, extra '%s'",palval);
  } else
  if (!strcmp(s,"parse error, unexpected ACTION, expecting NAME")) {
     printf("incorrect state name '%s'",palval);
  } else {
    printf("%s",s); 
  }
  printf("\n");
  return 1;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

int
yywrap()
{
  return 1;
}
