/**CFile*******************************************************************
  PackageName [strucval]
  Synopsis    [Structure-driven validation of hardware and software systems]

  FileName    [strucvalTcl.h]
  Revision    [$Revision: 2 $]
  Date        [$Date: 2006-10-05 15:30:29 +0200 (Thu, 05 Oct 2006) $]
  Authors     [Robert Meolic (meolic@uni-mb.si)]
  Description [File strucvalTcl.c contains definitions of Tcl commands.]
  SeeAlso     [strucval.h, strucvalInt.h]

  Copyright   [see strucval.h]
  ************************************************************************/

#include "strucvalInt.h"

/* on tcl 8.3 use #define USECONST */
/* on tcl 8.4 use #define USECONST const*/
/* this is defined in Makefile */

/*-----------------------------------------------------------------------*/
/* Constant declarations                                                 */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Variable declarations                                                 */
/*-----------------------------------------------------------------------*/

/**AutomaticStart*********************************************************/

/*-----------------------------------------------------------------------*/
/* Static function prototypes                                            */
/*-----------------------------------------------------------------------*/

static int StrucvalInitPkgCmd(ClientData clientData, Tcl_Interp *interp,
                        int argc, USECONST char **argv);

static int StrucvalAboutPkgCmd(ClientData clientData, Tcl_Interp *interp,
                         int argc, USECONST char **argv);

static int StrucvalSyncProductCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

/**AutomaticEnd***********************************************************/

/*-----------------------------------------------------------------------*/
/* Definition of exported functions                                      */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    [Function Strucval_Init.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

int
Strucval_Init(Tcl_Interp *interp)
{
  Tcl_CreateCommand(interp, "strucval_initPkg", StrucvalInitPkgCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "strucval_aboutPkg", StrucvalAboutPkgCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "strucval_sync_product", StrucvalSyncProductCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  return Tcl_PkgProvide(interp, "strucval", "1.0");
}

/*-----------------------------------------------------------------------*/
/* Definition of internal functions                                      */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Definition of static functions                                        */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    [Function StrucvalInitCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
StrucvalInitPkgCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                  USECONST char **argv)
{
  if (argc != 1) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  printf("Initialization of Strucval package... ");
  Strucval_InitPkg();
  printf("OK");

  printf("\n");
  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function StrucvalAboutPkgCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
StrucvalAboutPkgCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                   USECONST char **argv)
{
  if (argc != 1) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  Strucval_AboutPkg();

  printf("\n");
  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function StrucvalSyncProductCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
StrucvalSyncProductCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                    USECONST char **argv)
{
  Bdd_String s1,s2,s3,s4;

  if (argc != 5) {
    interp->result = "wrong # args";
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);
  s2 = strdup(argv[2]);
  s3 = strdup(argv[3]);
  s4 = strdup(argv[4]);

  Tcl_Eval(interp,"disable_input");
#ifdef USETHREADS
  Strucval_SyncProduct_Thr(s1,s2,s3,s4);
#else
  Strucval_SyncProduct(s1,s2,s3,s4);
#endif
  Tcl_Eval(interp,"enable_input");

  free(s1);
  free(s2);
  free(s3);
  free(s4);

  printf("\n");
  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}
