/**CFile*******************************************************************
  PackageName [versis]
  Synopsis    [Package 'versis' provides basic tools for formal
               verification of concurrent systems]

  FileName    [versisInt.h]
  Revision    [$Revision: 2 $]
  Date        [$Date: 2006-10-05 15:30:29 +0200 (Thu, 05 Oct 2006) $]
  Authors     [Robert Meolic (meolic@uni-mb.si),
               Mirjam Sepesy Maucec (mirjam.sepesy@uni-mb.si),
               Tatjana Kapus (kapus@uni-mb.si)]
  Description [File versisInt.h contains declaration of internal
               data structures.]
  SeeAlso     [versis.h]

  Copyright   [see versis.h]
  ************************************************************************/

#ifndef _VERSISINT
#define _VERSISINT

#ifndef TCLOUT
#  define Mi_Out printf
   char outthread;
   int outflag;
   char *out;
#else
#  include <mi.h>
#  define printf Mi_Printf
#endif

#include "versis.h"

/*-----------------------------------------------------------------------*/
/* Type declarations                                                     */
/*-----------------------------------------------------------------------*/

typedef int *TTabIndexov;   /* tabela indeksov spremenljivk */

/*-----------------------------------------------------------------------*/
/* Structure declarations                                                */
/*-----------------------------------------------------------------------*/

typedef struct {
  Bdd_Edge S;
  Bdd_Boolean b;
  int t;
} VersisST;

typedef struct {
  int num;
  VersisST *table;
} VersisStateTable;

/*-----------------------------------------------------------------------*/
/* Variable declarations                                                 */
/*-----------------------------------------------------------------------*/

/**AutomaticStart*********************************************************/

/*-----------------------------------------------------------------------*/
/* Function prototypes                                                   */
/*-----------------------------------------------------------------------*/

extern Bdd_Boolean VersisStrongEquivalence(Bdd_Edge *eqv,
                                      int par1, int n1, int par2, int n2);

extern Bdd_Boolean VersisWeakEquivalence(Bdd_Edge *eqv,
                                      int par1, int n1, int par2, int n2);

extern Bdd_Boolean VersisTestEquivalence(Bdd_Edge *eqv,
                                      int par1, int n1, int par2, int n2,
                                      int *acc1, int *acc2);

extern Bdd_Boolean VersisTraceEquivalence(int type, Bdd_Edge *eqv,
                                      int par1, int n1, int par2, int n2,
                                      int *det1, int *det2);

extern int VersisStrongMinimization(Bdd_String name, int par, int n);

extern int VersisWeakMinimization(Bdd_String name, int par, int n);

extern int VersisTraceMinimization(int type, Bdd_String name, int par, int n);

extern int VersisWCAMinimization(Bdd_String name, int par, int n);

extern Bdd_Edge VersisBuildDet(Bdd_String orgname, Bdd_String newname,
			Bdd_Edge init, Bdd_Edge Ptau, Bdd_Edge trcl, int s);

extern Bdd_Edge VersisTrCl(Bdd_Edge f);

extern Bdd_Edge VersisComposeActions(Pa_Composition *c);

/**AutomaticEnd***********************************************************/

#endif /* _VERSISINT */











