/**CFile*******************************************************************
  PackageName [versis]
  Synopsis    [Package 'versis' provides basic tools for formal
               verification of concurrent systems]

  FileName    [versisMain.c]
  Revision    [$Revision: 2 $]
  Date        [$Date: 2006-10-05 15:30:29 +0200 (Thu, 05 Oct 2006) $]
  Authors     [Robert Meolic (meolic@uni-mb.si),
               Mirjam Sepesy Maucec (mirjam.sepesy@uni-mb.si),
               Tatjana Kapus (kapus@uni-mb.si)]
  Description [File versisMain.c contains main functions.]
  SeeAlso     [versis.h, versisInt.h]

  Copyright   [see versis.h]
  ************************************************************************/

#include "versisInt.h"

/*-----------------------------------------------------------------------*/
/* Variable declarations                                                 */
/*-----------------------------------------------------------------------*/

int versis_status = 0;                     /* initialization status */

/**AutomaticStart*********************************************************/

/*-----------------------------------------------------------------------*/
/* Static function prototypes                                            */
/*-----------------------------------------------------------------------*/

/**AutomaticEnd***********************************************************/

/*-----------------------------------------------------------------------*/
/* Definition of exported functions                                      */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

#ifndef TCLOUT
   void Versis_Init(void) {}
#endif

/**Function****************************************************************
  Synopsis    [Function Versis_InitPkg.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
Versis_InitPkg()
{
  if (versis_status == 1) {
    printf("\nVersis package is already initialized. ");
    return;
  } else {
    versis_status = 1;
  }

  return;
}

/**Function****************************************************************
  Synopsis    [Function Versis_ExitPkg.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
Versis_ExitPkg()
{
}

/**Function****************************************************************
  Synopsis    [Function Versis_AboutPkg]
  Description [Versis_AboutPkg reports release of Versis package.]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
Versis_AboutPkg()
{
  printf("EST ");
  printf(EDITION);
  printf(", version ");
  printf(VERSION);
  printf(", ");
  printf("Versis package");
  printf("\n");
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
Versis_ChangeRename(Bdd_String process, Bdd_String name1, Bdd_String name2)
{
  int procnum;
  int act1,act2;
  Bdd_Edge substBA,subst,sup1,sup2,newtr;
  Pa_Process *p;
  int i;

  procnum = Pa_FindProcess(process);
  if (procnum == -1) {
    printf("Process does not exists.\n");
    return;
  }

  p = &pa_processTable[procnum];

  act1 =Pa_FindSortAction(&pa_sortTable[p->sort],name1);
  act2 =Pa_FindSortAction(&pa_sortTable[p->sort],name2);

  if ((act1 == -1) || (act2 == -1)) {
    printf("Action does not exists.\n");
    return;
  }

  if (act1 == 0) {
    printf("Cannot rename action TAU.\n");
    return;
  }

  if (p->tableS) {
    for (i=0; i<p->numTransitions; i++) {
      if (p->tableT[i].action == act1) {
        p->tableT[i].action = act2;
        if (act2 == 0) {
          p->tableT[i].type = 0;
	}
      }
    }
  }

  if (p->encoded) {

    newtr = Bdd_ITE(pa_sortTable[p->sort].table[act1].p,bdd_termFalse,p->d);

    sup1 = Bdd_ITE(pa_sortTable[p->sort].a0,
                     pa_sortTable[p->sort].table[act1].p,bdd_termFalse);
    sup2 = Bdd_ITE(pa_sortTable[p->sort].a0,
                     pa_sortTable[p->sort].table[act2].p,bdd_termFalse);
    subst = Bdd_RelOp(p->d,sup1,"#AND Ex xA",TRUE);
    sup2  = Bdd_ITE(subst,sup2,bdd_termFalse);
    newtr = Bdd_ITE(newtr,bdd_termTrue,sup2);

    sup1 = Bdd_ITE(pa_sortTable[p->sort].a0,bdd_termFalse,
                     pa_sortTable[p->sort].table[act1].p);
    sup2 = Bdd_ITE(pa_sortTable[p->sort].a0,bdd_termFalse,
                     pa_sortTable[p->sort].table[act2].p);
    subst = Bdd_RelOp(p->d,sup1,"#AND Ex xA",TRUE);
    sup2  = Bdd_ITE(subst,sup2,bdd_termFalse);
    newtr = Bdd_ITE(newtr,bdd_termTrue,sup2);

    p->d = newtr;
    Bdd_Fortify(p->d.p);
    Bdd_IncCounter();

    /* ****************** */
    /* ALTERNATIVE METHOD */
    /* ********************************************************************

    substBA = bdd_termFalse;
    for (i=0; i<pa_sortTable[p->sort].numActions; i++) {

      if (i>0) {
        sup1 = Bdd_ITE(pa_sortTable[p->sort].a0,
                       pa_sortTable[p->sort].table[i].p,bdd_termFalse);
        sup2 = sup1;
        sup1 = Bdd_RelOp(sup1,bdd_termFalse,"A2B",TRUE);

        if (i == act1) {
          sup2 = Bdd_ITE(pa_sortTable[p->sort].a0,
                         pa_sortTable[p->sort].table[act2].p,bdd_termFalse);
        }
        sup2 = Bdd_ITE(sup1,sup2,bdd_termFalse);
        substBA = Bdd_ITE(substBA,bdd_termTrue,sup2);
        Bdd_IncCounter();
        Bdd_Fresh(substBA.p);
      }

      sup1 = Bdd_ITE(pa_sortTable[p->sort].a0,bdd_termFalse,
                     pa_sortTable[p->sort].table[i].p);
      sup2 = sup1;
      sup1 = Bdd_RelOp(sup1,bdd_termFalse,"A2B",TRUE);

      if (i == act1) {
        sup2 = Bdd_ITE(pa_sortTable[p->sort].a0,bdd_termFalse,
                       pa_sortTable[p->sort].table[act2].p);
      }
      sup2 = Bdd_ITE(sup1,sup2,bdd_termFalse);
      substBA = Bdd_ITE(substBA,bdd_termTrue,sup2);
      Bdd_IncCounter();
      Bdd_Fresh(substBA.p);
    }

    newtr = Bdd_RelOp(p->d,bdd_termFalse,"A2B",TRUE);
    p->d = Bdd_RelOp(newtr,substBA,"#AND Ex xB",TRUE);
    Bdd_Fortify(p->d.p);
    Bdd_IncCounter();

    ******************************************************************** */

  }
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
Versis_ChangeHide(Bdd_String process, Bdd_String name1)
{
  int procnum;
  int act1;
  Pa_Process *p;
  int i;

  procnum = Pa_FindProcess(process);
  if (procnum == -1) {
    printf("Process does not exists.\n");
    return;
  }

  p = &pa_processTable[procnum];

  act1 =Pa_FindSortAction(&pa_sortTable[p->sort],name1);

  if (act1 == -1) {
    printf("Action does not exists.\n");
    return;
  }

  if (act1 == 0) {
    printf("Cannot hide action TAU.\n");
    return;
  }

  if (p->tableS) {
    for (i=0; i<p->numTransitions; i++) {
      if (p->tableT[i].action == act1) {
        p->tableT[i].action = 0;
        p->tableT[i].type = 0;
      }
    }
  }

  if (p->encoded) {
    p->encoded = 0;
    Pa_EncodeProcess(process);
  }
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

void
Versis_ChangeForbid(Bdd_String process, Bdd_String name1)
{
  int procnum;
  int act1;
  Pa_Process *p;
  Pa_Transition *table;
  int *states;
  int i,num;
  int encoded;

  procnum = Pa_FindProcess(process);
  if (procnum == -1) {
    printf("Process does not exists.\n");
    return;
  }

  p = &pa_processTable[procnum];

  act1 =Pa_FindSortAction(&pa_sortTable[p->sort],name1);

  if (act1 == -1) {
    printf("Action does not exists.\n");
    return;
  }

  if (p->compname) free(p->compname);
  p->compname = NULL;
  p->numVar = 0;
  p->d = bdd_termNull;
  p->stab = bdd_termNull;

  encoded = p->encoded;
  p->encoded = 0;

  num = p->numTransitions;
  p->numTransitions = 0;

  table = p->tableT;
  p->tableT = NULL;

  states = (int *) malloc(p->numStates * sizeof(int));
  for(i=0; i<p->numStates; i++) {
    states[i]=0;
  }
  states[p->initial]=1;

  if (table) {
    for (i=0; i<num; i++) {
      if (table[i].action != act1) {
        Pa_FOATransition(p,table[i].source,table[i].action,table[i].type,table[i].dest);
        states[table[i].dest]++;
      }
    }
  }

  for(i=0; i<p->numStates; i++) {
    if (!states[i]) Pa_DeleteStateProcess(p,i);
  }

  for(i=0; i<p->numStates; i++) {
    if (p->tableS[i].ccs) {
      free(p->tableS[i].ccs);
      p->tableS[i].ccs = NULL;
    }
  }

  free(table);
  free(states);

  if (encoded) {
    Pa_EncodeProcess(process);
  }
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

Bdd_Edge
Versis_CheckDeadlock(Bdd_String process)
{
  printf("\nversis_check_deadlock is not implemented.\n");

  return bdd_termNull;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

Bdd_Edge
Versis_CheckDivergence(Bdd_String process)
{
  printf("\nversis_check_divergence is not implemented.\n");

  return bdd_termNull;
}

/*-----------------------------------------------------------------------*/
/* Definition of internal functions                                      */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    [VersisTrCl]
  Description [Transitive closure on state variables RS. Not reflexive!]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

Bdd_Edge
VersisTrCl(Bdd_Edge f)
{
  Bdd_Edge f1,last,new,trcl;

  f1 = Bdd_R2P(f);
  last = bdd_termTrue;
  trcl = f;

  while (!Bdd_Cmp(trcl,last)) {

    Bdd_IncCounter();
    Bdd_Fresh(f1.p);
    Bdd_Fresh(trcl.p);

    last = trcl;
    new = Bdd_S2P(trcl);
    new = Bdd_ExP(new,f1);
    new = Bdd_ITE(new,bdd_termTrue,trcl);
    trcl = new;
  }

  return(trcl);
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

Bdd_Edge
VersisComposeActions(Pa_Composition *c)
{
  Pa_Sort *s;
  Bdd_Edge sup1,sup2;
  int i;

  s = &pa_sortTable[c->sort];
  sup1 = bdd_termFalse;

  for (i=0; i<c->numActions; i++) {
    sup2 = Bdd_ITE(s->a0,s->table[c->tableA[i]].p,bdd_termFalse);
    sup1 = Bdd_ITE(sup1,bdd_termTrue,sup2);
    sup2 = Bdd_ITE(s->a0,bdd_termFalse,s->table[c->tableA[i]].p);
    sup1 = Bdd_ITE(sup1,bdd_termTrue,sup2);
  }

  return sup1;
}

/*-----------------------------------------------------------------------*/
/* Definition of static functions                                        */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Functions, which are currently not used                               */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    []
  Description [Rezultat je presek vseh fun. STAB]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

Bdd_Edge
PRESEK_STAB(Pa_Composition *c)
{
  Bdd_Edge sup;
  int i;
 
  sup = bdd_termTrue;
  for (i=0; i<c->numProcesses; i++) {
    sup = Bdd_ITE(sup,pa_processTable[c->tableP[i]].stab,bdd_termFalse);
  }
  return(sup);
}

/**Function****************************************************************
  Synopsis    []
  Description [Rezultat je presek vseh fun. STAB, razen funkcij
               STAB i-tega]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

Bdd_Edge
PRESEK_STAB1(Pa_Composition *c, int index)
{
  Bdd_Edge sup;  
  int i;    
 
  sup = bdd_termTrue;
  for (i=0; i<c->numProcesses; i++) {
    if(i != index) {
      sup = Bdd_ITE(sup,pa_processTable[c->tableP[i]].stab,bdd_termFalse);
    }
  }
  return(sup);
}

/**Function****************************************************************
  Synopsis    []
  Description [Rezultat je presek vseh fun. STAB, razen funkcij STAB
               i-tega in j-tega procesa.]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

Bdd_Edge
PRESEK_STAB2(Pa_Composition *c, int index1, int index2)
{
  Bdd_Edge sup;  
  int i;    
 
  sup = bdd_termTrue;
  for (i=0; i<c->numProcesses; i++) {
    if((i != index1) && (i != index2)) {
      sup = Bdd_ITE(sup,pa_processTable[c->tableP[i]].stab,bdd_termFalse);
    }
  }
  return(sup);
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

double
SteviloStanj(Bdd_Edge S, int spr)
  {
    double Prestej(Bdd_Node *S, Bdd_Boolean pik, int spr)
      {
	if (S == bdd_termTrue.p)
	  if (pik)
	    return 0.0;
	  else
	    return pow(2.0, (double) spr);
	else
	  return Prestej(S->f.p, pik ^ S->f.mark, spr - 1) +
	         Prestej(S->t.p, pik ^ S->t.mark, spr - 1);
      }

    return Prestej(S.p, S.mark, spr);
  }
